@@ -1,6 +1,7 @@
+
 /*
  * OLSR ad-hoc routing table management protocol GUI front-end
- * Copyright (C) 2003 Andreas Tnnesen (andreto@ifi.uio.no)
+ * Copyright (C) 2003 Andreas Tonnesen (andreto@ifi.uio.no)
  *
  * This file is part of olsr.org.
  *
@@ -20,7 +21,6 @@
  *
  */
 
-
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -34,7 +34,6 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 
-
 GtkWidget *net_label;
 GtkWidget *info_label;
 GtkTextBuffer *textBuffer;
@@ -49,45 +48,29 @@ GtkWidget *mpr_list;
 GtkWidget *hna_list;
 GtkWidget *route_list;
 
+void fill_clist(GtkCList *);
 
-void
-fill_clist(GtkCList *);
+void selection_made(GtkWidget * clist, gint row, gint column, GdkEventButton * event, gpointer data);
 
+void set_net_info_connecting();
 
-void selection_made( GtkWidget      *clist,
-                     gint            row,
-                     gint            column,
-		     GdkEventButton *event,
-                     gpointer        data );
+void column_clicked_callback(GtkWidget *, gint);
 
+void connect_callback(GtkWidget * widget, gpointer data);
 
-void
-set_net_info_connecting();
+void packet_callback(GtkWidget * widget, gpointer data);
 
-void
-column_clicked_callback(GtkWidget *,gint);
+void packet_disp_callback(GtkWidget * widget, gpointer data);
 
+void packet_selection(GtkWidget * clist, gint row, gint column, GdkEventButton * event, gpointer data);
 
-void 
-connect_callback( GtkWidget *widget,
-		  gpointer   data );
+void node_selection(GtkWidget * clist, gint row, gint column, GdkEventButton * event, gpointer data);
 
-void 
-packet_callback( GtkWidget *widget,
-		  gpointer   data );
+void gui_shutdown(GtkObject *, gpointer);
 
-void 
-packet_disp_callback( GtkWidget *widget,
-		  gpointer   data );
-
-
-void
-packet_selection(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data);
-
-
-void
-node_selection(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer data);
-
-
-void
-gui_shutdown(GtkObject *, gpointer);
+/*
+ * Local Variables:
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * End:
+ */
