@@ -394,11 +394,12 @@ void RestoreSpoofFilter(void)
  * Description: Find the neighbors on a network interface to forward a BMF
  *              packet to
  * Input      : intf - the network interface
- *              source - the source IP address of the BMF packet 
+ *              source - the source IP address of the BMF packet, or NULL if
+ *                unknown or not applicable
  *              forwardedBy - the IP address of the node that forwarded the BMF
- *                packet
+ *                packet, or NULL if unknown or not applicable
  *              forwardedTo - the IP address of the node to which the BMF packet
- *                was directed
+ *                was directed, or NULL if unknown or not applicable
  * Output     : neighbors - list of (up to a number of 'FanOutLimit') neighbors.
  *              bestNeighbor - the best neighbor (in terms of lowest cost or ETX
  *                value)
@@ -414,6 +415,9 @@ void FindNeighbors(
   union olsr_ip_addr* forwardedTo,
   int* nPossibleNeighbors)
 {
+#ifndef NODEBUG
+  struct ipaddr_str buf;
+#endif
   int i;
 
   /* Initialize */
@@ -431,7 +435,6 @@ void FindNeighbors(
     struct link_entry* walker;
 
     OLSR_FOR_ALL_LINK_ENTRIES(walker) {
-      struct ipaddr_str buf;
       union olsr_ip_addr* neighborMainIp;
 
       /* Consider only links from the specified interface */
@@ -442,7 +445,7 @@ void FindNeighbors(
 
       OLSR_PRINTF(
         8,
-        "%s: ----> Considering forwarding pkt on \"%s\" to %s\n",
+        "%s: ----> considering forwarding pkt on \"%s\" to %s\n",
         PLUGIN_NAME_SHORT,
         intf->ifName,
         olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
@@ -453,10 +456,9 @@ void FindNeighbors(
        * passed IP addresses (if passed). Rely on short-circuit boolean evaluation. */
       if (source != NULL && ipequal(neighborMainIp, MainAddressOf(source)))
       {
-        struct ipaddr_str buf;
         OLSR_PRINTF(
           9,
-          "%s: ----> Not forwarding to %s: is source of pkt\n",
+          "%s: ----> not forwarding to %s: is source of pkt\n",
           PLUGIN_NAME_SHORT,
           olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
 
@@ -466,10 +468,9 @@ void FindNeighbors(
       /* Rely on short-circuit boolean evaluation */
       if (forwardedBy != NULL && ipequal(neighborMainIp, MainAddressOf(forwardedBy)))
       {
-        struct ipaddr_str buf;
         OLSR_PRINTF(
           9,
-          "%s: ----> Not forwarding to %s: is the node that forwarded the pkt\n",
+          "%s: ----> not forwarding to %s: is the node that forwarded the pkt\n",
           PLUGIN_NAME_SHORT,
           olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
 
@@ -479,10 +480,9 @@ void FindNeighbors(
       /* Rely on short-circuit boolean evaluation */
       if (forwardedTo != NULL && ipequal(neighborMainIp, MainAddressOf(forwardedTo)))
       {
-        struct ipaddr_str buf;
         OLSR_PRINTF(
           9,
-          "%s: ----> Not forwarding to %s: is the node to which the pkt was forwarded\n",
+          "%s: ----> not forwarding to %s: is the node to which the pkt was forwarded\n",
           PLUGIN_NAME_SHORT,
           olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
 
@@ -531,10 +531,8 @@ void FindNeighbors(
       }
     }
 
-    /* TODO: get_link_set() is not thread-safe! */
     for (walker = get_link_set(); walker != NULL; walker = walker->next) 
     {
-      struct ipaddr_str buf;
       union olsr_ip_addr* neighborMainIp;
       struct link_entry* bestLinkToNeighbor;
       struct tc_entry* tcLastHop;
@@ -547,7 +545,7 @@ void FindNeighbors(
 
       OLSR_PRINTF(
         9,
-        "%s: ----> Considering forwarding pkt on \"%s\" to %s\n",
+        "%s: ----> considering forwarding pkt on \"%s\" to %s\n",
         PLUGIN_NAME_SHORT,
         intf->ifName,
         olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
@@ -560,7 +558,7 @@ void FindNeighbors(
       {
         OLSR_PRINTF(
           9,
-          "%s: ----> Not forwarding to %s: is source of pkt\n",
+          "%s: ----> not forwarding to %s: is source of pkt\n",
           PLUGIN_NAME_SHORT,
           olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
 
@@ -572,7 +570,7 @@ void FindNeighbors(
       {
         OLSR_PRINTF(
           9,
-          "%s: ----> Not forwarding to %s: is the node that forwarded the pkt\n",
+          "%s: ----> not forwarding to %s: is the node that forwarded the pkt\n",
           PLUGIN_NAME_SHORT,
           olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
 
@@ -584,7 +582,7 @@ void FindNeighbors(
       {
         OLSR_PRINTF(
           9,
-          "%s: ----> Not forwarding to %s: is the node to which the pkt was forwarded\n",
+          "%s: ----> not forwarding to %s: is the node to which the pkt was forwarded\n",
           PLUGIN_NAME_SHORT,
           olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
 
@@ -597,7 +595,7 @@ void FindNeighbors(
       {
         OLSR_PRINTF(
           9,
-          "%s: ----> Not forwarding to %s: link is timing out\n",
+          "%s: ----> not forwarding to %s: link is timing out\n",
           PLUGIN_NAME_SHORT,
           olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
 
@@ -607,15 +605,14 @@ void FindNeighbors(
       /* Compare costs to check if the candidate neighbor is best reached via 'intf' */
       OLSR_PRINTF(
         9,
-        "%s: ----> Forwarding pkt to %s will cost %5.2f\n",
+        "%s: ----> forwarding pkt to %s will cost %5.2f\n",
         PLUGIN_NAME_SHORT,
         olsr_ip_to_string(&buf, &walker->neighbor_iface_addr),
         walker->link_cost);
 
       /* If the candidate neighbor is best reached via another interface, then skip 
        * the candidate neighbor; the candidate neighbor has been / will be selected via that
-       * other interface.
-       * TODO: get_best_link_to_neighbor() is not thread-safe. */
+       * other interface. */
       bestLinkToNeighbor = get_best_link_to_neighbor(&walker->neighbor_iface_addr);
 
       if (walker != bestLinkToNeighbor)
@@ -624,7 +621,7 @@ void FindNeighbors(
         {
           OLSR_PRINTF(
             9,
-            "%s: ----> Not forwarding to %s: no link found\n",
+            "%s: ----> not forwarding to %s: no link found\n",
             PLUGIN_NAME_SHORT,
             olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
         }
@@ -634,7 +631,7 @@ void FindNeighbors(
 
           OLSR_PRINTF(
             9,
-            "%s: ----> Not forwarding to %s: \"%s\" gives a better link to this neighbor, costing %5.2f\n",
+            "%s: ----> not forwarding to %s: \"%s\" gives a better link to this neighbor, costing %5.2f\n",
             PLUGIN_NAME_SHORT,
             olsr_ip_to_string(&buf, &walker->neighbor_iface_addr),
             bestIntf->int_name,
@@ -646,7 +643,6 @@ void FindNeighbors(
 
       if (forwardedBy != NULL)
       {
-        struct ipaddr_str forwardedByBuf, niaBuf;
         OLSR_PRINTF(
           9,
           "%s: ----> 2-hop path from %s via me to %s will cost %5.2f\n",
@@ -662,13 +658,11 @@ void FindNeighbors(
        * neighbor, because the 'forwardedBy' node will forward the packet. */
       if (forwardedBy != NULL)
       {
-        /* TODO: olsr_lookup_tc_entry() is not thread-safe. */
         tcLastHop = olsr_lookup_tc_entry(MainAddressOf(forwardedBy));
         if (tcLastHop != NULL)
         {
           struct tc_edge_entry* tc_edge;
 
-          /* TODO: olsr_lookup_tc_edge() is not thread-safe. */
           tc_edge = olsr_lookup_tc_edge(tcLastHop, MainAddressOf(&walker->neighbor_iface_addr));
           
           /* We are not interested in dead-end or dying edges. */
@@ -682,7 +676,7 @@ void FindNeighbors(
 #endif
               OLSR_PRINTF(
                 9,
-                "%s: ----> Not forwarding to %s: I am not an MPR between %s and %s, direct link costs %5.2f\n",
+                "%s: ----> not forwarding to %s: I am not an MPR between %s and %s, direct link costs %5.2f\n",
                 PLUGIN_NAME_SHORT,
                 neighbor_iface_buf.buf,
                 olsr_ip_to_string(&forw_buf, forwardedBy),
@@ -731,7 +725,6 @@ void FindNeighbors(
     }
 
     OLSR_FOR_ALL_LINK_ENTRIES(walker) {
-      struct ipaddr_str buf;
       union olsr_ip_addr* neighborMainIp;
       struct link_entry* bestLinkToNeighbor;
       struct tc_entry* tcLastHop;
@@ -745,7 +738,7 @@ void FindNeighbors(
 
       OLSR_PRINTF(
         9,
-        "%s: ----> Considering forwarding pkt on \"%s\" to %s\n",
+        "%s: ----> considering forwarding pkt on \"%s\" to %s\n",
         PLUGIN_NAME_SHORT,
         intf->ifName,
         olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
@@ -758,7 +751,7 @@ void FindNeighbors(
       {
         OLSR_PRINTF(
           9,
-          "%s: ----> Not forwarding to %s: is source of pkt\n",
+          "%s: ----> not forwarding to %s: is source of pkt\n",
           PLUGIN_NAME_SHORT,
           olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
 
@@ -770,7 +763,7 @@ void FindNeighbors(
       {
         OLSR_PRINTF(
           9,
-          "%s: ----> Not forwarding to %s: is the node that forwarded the pkt\n",
+          "%s: ----> not forwarding to %s: is the node that forwarded the pkt\n",
           PLUGIN_NAME_SHORT,
           olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
 
@@ -782,7 +775,7 @@ void FindNeighbors(
       {
         OLSR_PRINTF(
           9,
-          "%s: ----> Not forwarding to %s: is the node to which the pkt was forwarded\n",
+          "%s: ----> not forwarding to %s: is the node to which the pkt was forwarded\n",
           PLUGIN_NAME_SHORT,
           olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
 
@@ -794,11 +787,11 @@ void FindNeighbors(
       /* Calculate the link quality (ETX) of the link to the found neighbor */
       currEtx = walker->linkcost;
  
-      if (currEtx < LINK_COST_BROKEN)
+      if (currEtx >= LINK_COST_BROKEN)
       {
         OLSR_PRINTF(
           9,
-          "%s: ----> Not forwarding to %s: link is timing out\n",
+          "%s: ----> not forwarding to %s: link is timing out\n",
           PLUGIN_NAME_SHORT,
           olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
 
@@ -808,25 +801,23 @@ void FindNeighbors(
       /* Compare costs to check if the candidate neighbor is best reached via 'intf' */
       OLSR_PRINTF(
         9,
-        "%s: ----> Forwarding pkt to %s will cost ETX %5.2f\n",
+        "%s: ----> forwarding pkt to %s will cost ETX %5.2f\n",
         PLUGIN_NAME_SHORT,
         olsr_ip_to_string(&buf, &walker->neighbor_iface_addr),
         currEtx);
 
       /* If the candidate neighbor is best reached via another interface, then skip 
        * the candidate neighbor; the candidate neighbor has been / will be selected via that
-       * other interface.
-       * TODO: get_best_link_to_neighbor() is not thread-safe. */
+       * other interface. */
       bestLinkToNeighbor = get_best_link_to_neighbor(&walker->neighbor_iface_addr);
 
       if (walker != bestLinkToNeighbor)
       {
         if (bestLinkToNeighbor == NULL)
         {
-          struct ipaddr_str buf;
           OLSR_PRINTF(
             9,
-            "%s: ----> Not forwarding to %s: no link found\n",
+            "%s: ----> not forwarding to %s: no link found\n",
             PLUGIN_NAME_SHORT,
             olsr_ip_to_string(&buf, &walker->neighbor_iface_addr));
         }
@@ -834,16 +825,15 @@ void FindNeighbors(
         {
 #ifndef NODEBUG
           struct interface* bestIntf = if_ifwithaddr(&bestLinkToNeighbor->local_iface_addr);
-          struct ipaddr_str buf;
           struct lqtextbuffer lqbuffer;
 #endif
           OLSR_PRINTF(
             9,
-            "%s: ----> Not forwarding to %s: \"%s\" gives a better link to this neighbor, costing %s\n",
+            "%s: ----> not forwarding to %s: \"%s\" gives a better link to this neighbor, costing %s\n",
             PLUGIN_NAME_SHORT,
             olsr_ip_to_string(&buf, &walker->neighbor_iface_addr),
             bestIntf->int_name,
-            get_linkcost_text(bestLinkToNeighbor->linkcost, OLSR_FALSE, &lqbuffer));
+            get_linkcost_text(bestLinkToNeighbor->linkcost, false, &lqbuffer));
         }
 
         continue; /* for */
@@ -861,7 +851,7 @@ void FindNeighbors(
           PLUGIN_NAME_SHORT,
           olsr_ip_to_string(&forwardedByBuf, forwardedBy),
           olsr_ip_to_string(&niaBuf, &walker->neighbor_iface_addr),
-          get_linkcost_text(previousLinkEtx + currEtx, OLSR_TRUE, &lqbuffer));
+          get_linkcost_text(previousLinkEtx + currEtx, true, &lqbuffer));
       }
 
       /* Check the topology table whether the 'forwardedBy' node is itself a direct
@@ -870,13 +860,11 @@ void FindNeighbors(
        * neighbor, because the 'forwardedBy' node will forward the packet. */
       if (forwardedBy != NULL)
       {
-        /* TODO: olsr_lookup_tc_entry() is not thread-safe. */
         tcLastHop = olsr_lookup_tc_entry(MainAddressOf(forwardedBy));
         if (tcLastHop != NULL)
         {
           struct tc_edge_entry* tc_edge;
 
-          /* TODO: olsr_lookup_tc_edge() is not thread-safe. */
           tc_edge = olsr_lookup_tc_edge(tcLastHop, MainAddressOf(&walker->neighbor_iface_addr));
 
           /* We are not interested in dead-end edges. */
@@ -892,12 +880,12 @@ void FindNeighbors(
 #endif
               OLSR_PRINTF(
                 9,
-                "%s: ----> Not forwarding to %s: I am not an MPR between %s and %s, direct link costs %s\n",
+                "%s: ----> not forwarding to %s: I am not an MPR between %s and %s, direct link costs %s\n",
                 PLUGIN_NAME_SHORT,
                 neighbor_iface_buf.buf,
                 olsr_ip_to_string(&forw_buf, forwardedBy),
                 neighbor_iface_buf.buf,
-                get_linkcost_text(tcEtx, OLSR_FALSE, &lqbuffer));
+                get_linkcost_text(tcEtx, false, &lqbuffer));
 
               continue; /* for */
             } /* if */
@@ -932,13 +920,12 @@ void FindNeighbors(
   {
     OLSR_PRINTF(
       9,
-      "%s: ----> No suitable neighbor found to forward to on \"%s\"\n",
+      "%s: ----> no suitable neighbor found to forward to on \"%s\"\n",
       PLUGIN_NAME_SHORT,
       intf->ifName);
   }
   else
   {
-    struct ipaddr_str buf;
     OLSR_PRINTF(
       9,
       "%s: ----> %d neighbors found on \"%s\"; best neighbor to forward to: %s\n",
@@ -1429,6 +1416,16 @@ static int CreateInterface(
     return 0;
   }
 
+  /* add listeners to sockets */
+  if (capturingSkfd != -1) {
+    add_olsr_socket(capturingSkfd, NULL, BMF_handle_captureFd, newIf, SP_IMM_READ);
+  }
+  if (encapsulatingSkfd != -1) {
+    add_olsr_socket(encapsulatingSkfd, NULL, BMF_handle_encapsulatingFd, newIf, SP_IMM_READ);
+  }
+  if (listeningSkfd != -1) {
+    add_olsr_socket(listeningSkfd, NULL, BMF_handle_listeningFd, newIf, SP_IMM_READ);
+  }
   /* Copy data into TBmfInterface object */
   newIf->capturingSkfd = capturingSkfd;
   newIf->encapsulatingSkfd = encapsulatingSkfd;
@@ -1479,10 +1476,6 @@ static int CreateInterface(
     }
   }
 
-  /* Initialize fragment history table */
-  memset(&newIf->fragmentHistory, 0, sizeof(newIf->fragmentHistory));
-  newIf->nextFragmentHistoryEntry = 0;
-
   /* Reset counters */
   newIf->nBmfPacketsRx = 0;
   newIf->nBmfPacketsRxDup = 0;
@@ -1493,6 +1486,7 @@ static int CreateInterface(
   if (BmfInterfaces == NULL)
   {
     /* First TBmfInterface object in list */
+    newIf->next = NULL;
     BmfInterfaces = newIf;
     LastBmfInterface = newIf;
   }
@@ -1616,6 +1610,7 @@ int CreateBmfNetworkInterfaces(struct interface* skipThisIntf)
   EtherTunTapFd = CreateLocalEtherTunTap();
   if (EtherTunTapFd >= 0)
   {
+    add_olsr_socket(EtherTunTapFd, NULL, BMF_handle_tuntapFd, NULL, SP_IMM_READ);
     nOpenedSockets++;
   }
 
@@ -1630,33 +1625,13 @@ int CreateBmfNetworkInterfaces(struct interface* skipThisIntf)
   return 0;
 } /* CreateBmfNetworkInterfaces */
 
-/* -------------------------------------------------------------------------
- * Function   : AddInterface
- * Description: Add an OLSR-enabled network interface to the list of BMF-enabled
- *              network interfaces
- * Input      : newIntf - network interface to add
- * Output     : none
- * Return     : none
- * Data Used  : none
- * ------------------------------------------------------------------------- */
-void AddInterface(struct interface* newIntf)
-{
-  int nOpened;
-
-  assert(newIntf != NULL);
-
-  nOpened = CreateInterface(newIntf->int_name, newIntf);
-
-  olsr_printf(1, "%s: opened %d sockets\n", PLUGIN_NAME, nOpened);
-} /* AddInterface */
-
 /* -------------------------------------------------------------------------
  * Function   : CloseBmfNetworkInterfaces
  * Description: Closes every socket on each network interface used by BMF
  * Input      : none
  * Output     : none
  * Return     : none
- * Data Used  : none
+ * Data Used  : BmfInterfaces, LastBmfInterface
  * Notes      : Closes
  *              - the local EtherTunTap interface (e.g. "tun0" or "tap0")
  *              - for each BMF-enabled interface, the socket used for
@@ -1686,11 +1661,19 @@ void CloseBmfNetworkInterfaces(void)
     if (bmfIf->capturingSkfd >= 0)
     {
       close(bmfIf->capturingSkfd);
+      remove_olsr_socket(bmfIf->capturingSkfd, NULL, BMF_handle_captureFd);
       nClosed++;
     }
     if (bmfIf->encapsulatingSkfd >= 0) 
     {
       close(bmfIf->encapsulatingSkfd);
+      remove_olsr_socket(bmfIf->encapsulatingSkfd, NULL, BMF_handle_encapsulatingFd);
+      nClosed++;
+    }
+    if (bmfIf->listeningSkfd >= 0)
+    {
+      close(bmfIf->listeningSkfd);
+      remove_olsr_socket(bmfIf->listeningSkfd, NULL, BMF_handle_listeningFd);
       nClosed++;
     }
 
@@ -1731,25 +1714,31 @@ void CloseBmfNetworkInterfaces(void)
   if (EtherTunTapFd >= 0)
   {
     close(EtherTunTapFd);
+    remove_olsr_socket(EtherTunTapFd, NULL, BMF_handle_tuntapFd);
     nClosed++;
 
     OLSR_PRINTF(7, "%s: closed \"%s\"\n", PLUGIN_NAME_SHORT, EtherTunTapIfName);
   }
 
   BmfInterfaces = NULL;
+  LastBmfInterface = NULL;
+
+  /* Re-initialize the IP address for the BMF network interface. Thanks to
+   * Daniele Lacamera for finding and solving this bug. */
+  EtherTunTapIp = ETHERTUNTAPIPNOTSET;
 
   olsr_printf(1, "%s: closed %d sockets\n", PLUGIN_NAME_SHORT, nClosed);
 
   OLSR_PRINTF(
     7,
-    "%s: Total all OLSR interfaces    : RX pkts %d (%d dups); TX pkts %d\n",
+    "%s: total all OLSR interfaces    : RX pkts %d (%d dups); TX pkts %d\n",
     PLUGIN_NAME_SHORT,
     totalOlsrBmfPacketsRx,
     totalOlsrBmfPacketsRxDup,
     totalOlsrBmfPacketsTx);
   OLSR_PRINTF(
     7,
-    "%s: Total all non-OLSR interfaces: RX pkts %d (%d dups); TX pkts %d\n",
+    "%s: total all non-OLSR interfaces: RX pkts %d (%d dups); TX pkts %d\n",
     PLUGIN_NAME_SHORT,
     totalNonOlsrBmfPacketsRx,
     totalNonOlsrBmfPacketsRxDup,
