@@ -1,3 +1,4 @@
+
 /* MD5.H - header file for MD5C.C
  */
 
@@ -12,37 +13,47 @@
    License is also granted to make and use derivative works provided
    that such works are identified as "derived from the RSA Data
    Security, Inc. MD5 Message-Digest Algorithm" in all material
-   mentioning or referencing the derived work.  
-                                                                    
+   mentioning or referencing the derived work.
+
    RSA Data Security, Inc. makes no representations concerning either
    the merchantability of this software or the suitability of this
    software for any particular purpose. It is provided "as is"
-   without express or implied warranty of any kind.  
-                                                                    
+   without express or implied warranty of any kind.
+
    These notices must be retained in any copies of any part of this
-   documentation and/or software.  
+   documentation and/or software.
  */
 
 #ifndef _MD5_H_
 #define _MD5_H_
 
+#include <inttypes.h>
+
 /* POINTER defines a generic pointer type */
-typedef unsigned char *POINTER;
+typedef uint8_t *POINTER;
+
 /* UINT2 defines a two byte word */
-typedef unsigned short int UINT2;
+typedef uint16_t UINT2;
+
 /* UINT4 defines a four byte word */
-typedef unsigned long int UINT4;
+typedef uint32_t UINT4;
 
 /* MD5 context. */
 typedef struct {
-  UINT4 state[4];                                   /* state (ABCD) */
-  UINT4 count[2];        /* number of bits, modulo 2^64 (lsb first) */
-  unsigned char buffer[64];                         /* input buffer */
+  UINT4 state[4];                      /* state (ABCD) */
+  UINT4 count[2];                      /* number of bits, modulo 2^64 (lsb first) */
+  unsigned char buffer[64];            /* input buffer */
 } MD5_CTX;
 
 void MD5Init(MD5_CTX *);
 void MD5Update(MD5_CTX *, const unsigned char *, const unsigned int);
-void MD5Final(unsigned char [16], MD5_CTX *);
-
+void MD5Final(unsigned char[16], MD5_CTX *);
 
 #endif
+
+/*
+ * Local Variables:
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * End:
+ */
