@@ -1,9 +1,9 @@
+
 /*
 ** Interface to Memory Manager
 ** See Copyright Notice in lua.h
 */
 
-
 #include <stdlib.h>
 
 #define lmem_c
@@ -16,8 +16,6 @@
 #include "lobject.h"
 #include "lstate.h"
 
-
-
 /*
 ** definition for realloc function. It must assure that l_realloc(NULL,
 ** 0, x) allocates a new block (ANSI C assures that). (`os' is the old
@@ -35,49 +33,49 @@
 #define l_free(b,os)	free(b)
 #endif
 
-
 #define MINSIZEARRAY	4
 
-
-void *luaM_growaux (lua_State *L, void *block, int *size, int size_elems,
-                    int limit, const char *errormsg) {
+void *
+luaM_growaux(lua_State * L, void *block, int *size, int size_elems, int limit, const char *errormsg)
+{
   void *newblock;
-  int newsize = (*size)*2;
+  int newsize = (*size) * 2;
   if (newsize < MINSIZEARRAY)
-    newsize = MINSIZEARRAY;  /* minimum size */
-  else if (*size >= limit/2) {  /* cannot double it? */
-    if (*size < limit - MINSIZEARRAY)  /* try something smaller... */
-      newsize = limit;  /* still have at least MINSIZEARRAY free places */
-    else luaG_runerror(L, errormsg);
+    newsize = MINSIZEARRAY;     /* minimum size */
+  else if (*size >= limit / 2) {        /* cannot double it? */
+    if (*size < limit - MINSIZEARRAY)   /* try something smaller... */
+      newsize = limit;          /* still have at least MINSIZEARRAY free places */
+    else
+      luaG_runerror(L, errormsg);
   }
-  newblock = luaM_realloc(L, block,
-                          cast(lu_mem, *size)*cast(lu_mem, size_elems),
-                          cast(lu_mem, newsize)*cast(lu_mem, size_elems));
-  *size = newsize;  /* update only when everything else is OK */
+  newblock =
+    luaM_realloc(L, block, cast(lu_mem, *size) * cast(lu_mem, size_elems), cast(lu_mem, newsize) * cast(lu_mem, size_elems));
+  *size = newsize;              /* update only when everything else is OK */
   return newblock;
 }
 
-
 /*
 ** generic allocation routine.
 */
-void *luaM_realloc (lua_State *L, void *block, lu_mem oldsize, lu_mem size) {
+void *
+luaM_realloc(lua_State * L, void *block, lu_mem oldsize, lu_mem size)
+{
   lua_assert((oldsize == 0) == (block == NULL));
   if (size == 0) {
     if (block != NULL) {
       l_free(block, oldsize);
       block = NULL;
-    }
-    else return NULL;  /* avoid `nblocks' computations when oldsize==size==0 */
-  }
-  else if (size >= MAX_SIZET)
+    } else
+      return NULL;              /* avoid `nblocks' computations when oldsize==size==0 */
+  } else if (size >= MAX_SIZET)
     luaG_runerror(L, "memory allocation error: block too big");
   else {
     block = l_realloc(block, oldsize, size);
     if (block == NULL) {
       if (L)
         luaD_throw(L, LUA_ERRMEM);
-      else return NULL;  /* error before creating state! */
+      else
+        return NULL;            /* error before creating state! */
     }
   }
   if (L) {
@@ -88,3 +86,9 @@ void *luaM_realloc (lua_State *L, void *block, lu_mem oldsize, lu_mem size) {
   return block;
 }
 
+/*
+ * Local Variables:
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * End:
+ */
