@@ -1,3 +1,4 @@
+
 /*
 ** Interface to Memory Manager
 ** See Copyright Notice in lua.h
@@ -6,7 +7,6 @@
 #ifndef lmem_h
 #define lmem_h
 
-
 #include <stddef.h>
 
 #include "llimits.h"
@@ -14,11 +14,9 @@
 
 #define MEMERRMSG	"not enough memory"
 
+void *luaM_realloc(lua_State * L, void *oldblock, lu_mem oldsize, lu_mem size);
 
-void *luaM_realloc (lua_State *L, void *oldblock, lu_mem oldsize, lu_mem size);
-
-void *luaM_growaux (lua_State *L, void *block, int *size, int size_elem,
-                    int limit, const char *errormsg);
+void *luaM_growaux(lua_State * L, void *block, int *size, int size_elem, int limit, const char *errormsg);
 
 #define luaM_free(L, b, s)	luaM_realloc(L, (b), (s), 0)
 #define luaM_freelem(L, b)	luaM_realloc(L, (b), sizeof(*(b)), 0)
@@ -38,6 +36,11 @@ void *luaM_growaux (lua_State *L, void *block, int *size, int size_elem,
    ((v)=cast(t *, luaM_realloc(L, v,cast(lu_mem, oldn)*cast(lu_mem, sizeof(t)), \
                                     cast(lu_mem, n)*cast(lu_mem, sizeof(t)))))
 
-
 #endif
 
+/*
+ * Local Variables:
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * End:
+ */
