@@ -62,21 +62,21 @@ struct link_entry {
   char *if_name;
   struct timer_entry *link_timer;
   struct timer_entry *link_sym_timer;
-  clock_t ASYM_time;
+  uint32_t ASYM_time;
   olsr_reltime vtime;
   struct neighbor_entry *neighbor;
-  olsr_u8_t prev_status;
+  uint8_t prev_status;
 
   /*
    * Hysteresis
    */
   float L_link_quality;
   int L_link_pending;
-  clock_t L_LOST_LINK_time;
+  uint32_t L_LOST_LINK_time;
   struct timer_entry *link_hello_timer; /* When we should receive a new HELLO */
   olsr_reltime last_htime;
-  olsr_bool olsr_seqno_valid;
-  olsr_u16_t olsr_seqno;
+  bool olsr_seqno_valid;
+  uint16_t olsr_seqno;
 
   /*
    * packet loss
@@ -85,13 +85,13 @@ struct link_entry {
   struct timer_entry *link_loss_timer;
 
   /* user defined multiplies for link quality, multiplied with 65536 */
-  olsr_u32_t loss_link_multiplier;
+  uint32_t loss_link_multiplier;
 
   /* cost of this link */
   olsr_linkcost linkcost;
 
   struct list_node link_list;          /* double linked list of all link entries */
-  olsr_u32_t linkquality[0];
+  uint32_t linkquality[0];
 };
 
 /* inline to recast from link_list back to link_entry */
@@ -115,16 +115,16 @@ LISTNODE2STRUCT(list2link, struct link_entry, link_list);
 
 /* Externals */
 extern struct list_node link_entry_head;
-extern olsr_bool link_changes;
+extern bool link_changes;
 
 /* Function prototypes */
 
 void olsr_set_link_timer(struct link_entry *, unsigned int);
 void olsr_init_link_set(void);
+void olsr_reset_all_links(void);
 void olsr_delete_link_entry_by_ip(const union olsr_ip_addr *);
 void olsr_expire_link_hello_timer(void *);
-void olsr_update_packet_loss_worker(struct link_entry *, olsr_bool);
-void signal_link_changes(olsr_bool);   /* XXX ugly */
+void signal_link_changes(bool);        /* XXX ugly */
 
 struct link_entry *get_best_link_to_neighbor(const union olsr_ip_addr *);
 
@@ -137,7 +137,7 @@ int check_neighbor_link(const union olsr_ip_addr *);
 int replace_neighbor_link_set(const struct neighbor_entry *, struct neighbor_entry *);
 int lookup_link_status(const struct link_entry *);
 void olsr_update_packet_loss_hello_int(struct link_entry *, olsr_reltime);
-void olsr_update_packet_loss(struct link_entry *entry);
+void olsr_received_hello_handler(struct link_entry *entry);
 void olsr_print_link_set(void);
 
 #endif
