@@ -53,7 +53,14 @@
 
 void olsr_openlog(const char *ident);
 
+#if SYSLOG_NUMBERING
+void olsr_syslog_real(int level, const char *format, ...) __attribute__ ((format(printf, 2, 3)));
+
+extern unsigned int olsr_syslog_ctr;
+#define olsr_syslog(level, format, ...) olsr_syslog_real (level, "%02x: " format, olsr_syslog_ctr++ & 0xFF, ## __VA_ARGS__ )
+#else
 void olsr_syslog(int level, const char *format, ...) __attribute__ ((format(printf, 2, 3)));
+#endif
 
 #endif
 
