@@ -93,8 +93,8 @@ olsr_del_nbr2_list(struct neighbor_2_list_entry *nbr2_list)
   free(nbr2_list);
 
   /* Set flags to recalculate the MPR set and the routing table */
-  changes_neighborhood = OLSR_TRUE;
-  changes_topology = OLSR_TRUE;
+  changes_neighborhood = true;
+  changes_topology = true;
 }
 
 /**
@@ -106,14 +106,14 @@ olsr_del_nbr2_list(struct neighbor_2_list_entry *nbr2_list)
  * @return positive if entry deleted
  */
 int
-olsr_delete_neighbor_2_pointer(struct neighbor_entry *neighbor, union olsr_ip_addr *address)
+olsr_delete_neighbor_2_pointer(struct neighbor_entry *neighbor, struct neighbor_2_entry *neigh2)
 {
   struct neighbor_2_list_entry *nbr2_list;
 
   nbr2_list = neighbor->neighbor_2_list.next;
 
   while (nbr2_list != &neighbor->neighbor_2_list) {
-    if (ipequal(&nbr2_list->neighbor_2->neighbor_2_addr, address)) {
+    if (nbr2_list->neighbor_2 == neigh2) {
       olsr_del_nbr2_list(nbr2_list);
       return 1;
     }
@@ -147,6 +147,24 @@ olsr_lookup_my_neighbors(const struct neighbor_entry *neighbor, const union olsr
   return NULL;
 }
 
+/**
+ * Update a neighbours main_addr inlcuding hash
+*/
+
+void
+olsr_update_neighbor_main_addr(struct neighbor_entry *entry, const union olsr_ip_addr *new_main_addr)
+{
+  /*remove from old pos*/
+  DEQUEUE_ELEM(entry);
+
+  /*update main addr*/
+  entry->neighbor_main_addr = *new_main_addr;
+
+  /*insert it again*/
+  QUEUE_ELEM(neighbortable[olsr_ip_hashing(new_main_addr)], entry);
+
+}
+
 /**
  *Delete a neighbr table entry.
  *
@@ -161,7 +179,7 @@ int
 olsr_delete_neighbor_table(const union olsr_ip_addr *neighbor_addr)
 {
   struct neighbor_2_list_entry *two_hop_list, *two_hop_to_delete;
-  olsr_u32_t hash;
+  uint32_t hash;
   struct neighbor_entry *entry;
 
   //printf("inserting neighbor\n");
@@ -190,7 +208,7 @@ olsr_delete_neighbor_table(const union olsr_ip_addr *neighbor_addr)
     two_hop_list = two_hop_list->next;
 
     two_hop_to_delete->neighbor_2->neighbor_2_pointer--;
-    olsr_delete_neighbor_pointer(two_hop_to_delete->neighbor_2, &entry->neighbor_main_addr);
+    olsr_delete_neighbor_pointer(two_hop_to_delete->neighbor_2, entry);
 
     olsr_del_nbr2_list(two_hop_to_delete);
   }
@@ -200,7 +218,7 @@ olsr_delete_neighbor_table(const union olsr_ip_addr *neighbor_addr)
 
   free(entry);
 
-  changes_neighborhood = OLSR_TRUE;
+  changes_neighborhood = true;
   return 1;
 
 }
@@ -215,7 +233,7 @@ olsr_delete_neighbor_table(const union olsr_ip_addr *neighbor_addr)
 struct neighbor_entry *
 olsr_insert_neighbor_table(const union olsr_ip_addr *main_addr)
 {
-  olsr_u32_t hash;
+  uint32_t hash;
   struct neighbor_entry *new_neigh;
 
   hash = olsr_ip_hashing(main_addr);
@@ -240,8 +258,8 @@ olsr_insert_neighbor_table(const union olsr_ip_addr *main_addr)
   new_neigh->neighbor_2_list.prev = &new_neigh->neighbor_2_list;
 
   new_neigh->linkcount = 0;
-  new_neigh->is_mpr = OLSR_FALSE;
-  new_neigh->was_mpr = OLSR_FALSE;
+  new_neigh->is_mpr = false;
+  new_neigh->was_mpr = false;
 
   /* Queue */
   QUEUE_ELEM(neighbortable[hash], new_neigh);
@@ -281,7 +299,7 @@ struct neighbor_entry *
 olsr_lookup_neighbor_table_alias(const union olsr_ip_addr *dst)
 {
   struct neighbor_entry *entry;
-  olsr_u32_t hash = olsr_ip_hashing(dst);
+  uint32_t hash = olsr_ip_hashing(dst);
 
   //printf("\nLookup %s\n", olsr_ip_to_string(&buf, dst));
   for (entry = neighbortable[hash].next; entry != &neighbortable[hash]; entry = entry->next) {
@@ -313,18 +331,18 @@ update_neighbor_status(struct neighbor_entry *entry, int lnk)
         olsr_delete_two_hop_neighbor_table(two_hop_neighbor);
       }
 
-      changes_neighborhood = OLSR_TRUE;
-      changes_topology = OLSR_TRUE;
+      changes_neighborhood = true;
+      changes_topology = true;
       if (olsr_cnf->tc_redundancy > 1)
-        signal_link_changes(OLSR_TRUE);
+        signal_link_changes(true);
     }
     entry->status = SYM;
   } else {
     if (entry->status == SYM) {
-      changes_neighborhood = OLSR_TRUE;
-      changes_topology = OLSR_TRUE;
+      changes_neighborhood = true;
+      changes_topology = true;
       if (olsr_cnf->tc_redundancy > 1)
-        signal_link_changes(OLSR_TRUE);
+        signal_link_changes(true);
     }
     /* else N_status is set to NOT_SYM */
     entry->status = NOT_SYM;
@@ -351,7 +369,7 @@ olsr_expire_nbr2_list(void *context)
   nbr2 = nbr2_list->neighbor_2;
 
   nbr2->neighbor_2_pointer--;
-  olsr_delete_neighbor_pointer(nbr2, &nbr->neighbor_main_addr);
+  olsr_delete_neighbor_pointer(nbr2, nbr);
 
   olsr_del_nbr2_list(nbr2_list);
 }
