@@ -43,69 +43,77 @@
 #define _OLSRD_CFGPARSER_H
 
 #include "olsr_types.h"
+#include "common/autobuf.h"
+
+/* set to 1 to collect all startup sleep into one sleep
+ * (just as long as the longest sleep)
+ * useful if many errors on many interfaces */
+#define OLSR_COLLECT_STARTUP_SLEEP 1
 
 #define TESTLIB_PATH 0
 #define SYSLOG_NUMBERING 0
-#define SOURCE_IP_ROUTES 0
-
-#ifndef LINUX_POLICY_ROUTING
-#if defined linux
-#  define LINUX_POLICY_ROUTING 1
-#else
-#  define LINUX_POLICY_ROUTING 0
-#endif
-#endif
 
 /* Default values not declared in olsr_protocol.h */
-#define DEF_POLLRATE        0.05
-#define DEF_NICCHGPOLLRT    2.5
-#define DEF_WILL_AUTO       true
-#define DEF_ALLOW_NO_INTS   true
-#define DEF_TOS             16
-#define DEF_DEBUGLVL        1
-#define DEF_IPC_CONNECTIONS 0
-#define DEF_USE_HYST        false
-#define DEF_FIB_METRIC      FIBM_FLAT
-#define DEF_LQ_LEVEL        2
-#define DEF_LQ_FISH         0
-#define DEF_LQ_DIJK_LIMIT   255
-#define DEF_LQ_DIJK_INTER   0.0
-#define DEF_LQ_NAT_THRESH   1.0
-#define DEF_LQ_AGING        0.1
-#define DEF_CLEAR_SCREEN    false
-#define DEF_OLSRPORT        698
-#define DEF_RTPROTO         0 /* 0 means OS-specific default */
-#define DEF_RTTABLE         254
+#define DEF_IP_VERSION       AF_INET
+#define DEF_POLLRATE         0.05
+#define DEF_NICCHGPOLLRT     2.5
+#define DEF_WILL_AUTO        false
+#define DEF_WILLINGNESS      3
+#define DEF_ALLOW_NO_INTS    true
+#define DEF_TOS              16
+#define DEF_DEBUGLVL         1
+#define DEF_IPC_CONNECTIONS  0
+#define DEF_USE_HYST         false
+#define DEF_FIB_METRIC       FIBM_FLAT
+#define DEF_LQ_LEVEL         2
+#define DEF_LQ_ALGORITHM     "etx_ff"
+#define DEF_LQ_FISH          1
+#define DEF_LQ_NAT_THRESH    1.0
+#define DEF_LQ_AGING         0.05
+#define DEF_CLEAR_SCREEN     true
+#define DEF_OLSRPORT         698
+#define DEF_RTPROTO          0 /* 0 means OS-specific default */
+#define DEF_RT_NONE          -1
+#define DEF_RT_AUTO          0
+#define DEF_MIN_TC_VTIME     0.0
+#define DEF_USE_NIIT         true
+#define DEF_SMART_GW         false
+#define DEF_GW_ALLOW_NAT     true
+#define DEF_GW_TYPE          GW_UPLINK_IPV46
+#define DEF_GW_UPLINK_NAT    true
+#define DEF_UPLINK_SPEED     128
+#define DEF_DOWNLINK_SPEED   1024
+#define DEF_USE_SRCIP_ROUTES false
+
+#define DEF_IF_MODE          IF_MODE_MESH
 
 /* Bounds */
 
-#define MIN_INTERVAL        0.01
+#define MIN_INTERVAL         0.01
 
-#define MAX_POLLRATE        1.0
-#define MIN_POLLRATE        0.01
-#define MAX_NICCHGPOLLRT    100.0
-#define MIN_NICCHGPOLLRT    1.0
-#define MAX_DEBUGLVL        9
-#define MIN_DEBUGLVL        0
-#define MAX_TOS             16
-#define MIN_TOS             0
-#define MAX_WILLINGNESS     7
-#define MIN_WILLINGNESS     0
-#define MAX_MPR_COVERAGE    20
-#define MIN_MPR_COVERAGE    1
-#define MAX_TC_REDUNDANCY   2
-#define MIN_TC_REDUNDANCY   0
-#define MAX_HYST_PARAM      1.0
-#define MIN_HYST_PARAM      0.0
-#define MAX_LQ_LEVEL        2
-#define MIN_LQ_LEVEL        0
-#define MAX_LQ_AGING        1.0
-#define MIN_LQ_AGING        0.01
+#define MAX_POLLRATE         1.0
+#define MIN_POLLRATE         0.01
+#define MAX_NICCHGPOLLRT     100.0
+#define MIN_NICCHGPOLLRT     1.0
+#define MAX_DEBUGLVL         9
+#define MIN_DEBUGLVL         0
+#define MAX_TOS              16
+#define MIN_TOS              0
+#define MAX_WILLINGNESS      7
+#define MIN_WILLINGNESS      0
+#define MAX_MPR_COVERAGE     20
+#define MIN_MPR_COVERAGE     1
+#define MAX_TC_REDUNDANCY    2
+#define MIN_TC_REDUNDANCY    0
+#define MAX_HYST_PARAM       1.0
+#define MIN_HYST_PARAM       0.0
+#define MAX_LQ_LEVEL         2
+#define MIN_LQ_LEVEL         0
+#define MAX_LQ_AGING         1.0
+#define MIN_LQ_AGING         0.01
 
-/* Option values */
-#define CFG_FIBM_FLAT          "flat"
-#define CFG_FIBM_CORRECT       "correct"
-#define CFG_FIBM_APPROX        "approx"
+#define MIN_SMARTGW_SPEED    1
+#define MAX_SMARTGW_SPEED    320000000
 
 #ifndef IPV6_ADDR_SITELOCAL
 #define IPV6_ADDR_SITELOCAL    0x0040U
@@ -113,6 +121,29 @@
 
 #include "interfaces.h"
 
+enum smart_gw_uplinktype {
+  GW_UPLINK_NONE,
+  GW_UPLINK_IPV4,
+  GW_UPLINK_IPV6,
+  GW_UPLINK_IPV46,
+  GW_UPLINK_CNT,
+};
+
+
+typedef enum {
+  FIBM_FLAT,
+  FIBM_CORRECT,
+  FIBM_APPROX,
+  FIBM_CNT
+} olsr_fib_metric_options;
+
+enum olsr_if_mode {
+  IF_MODE_MESH,
+  IF_MODE_ETHER,
+  IF_MODE_CNT
+};
+
+
 struct olsr_msg_params {
   float emission_interval;
   float validity_time;
@@ -130,28 +161,31 @@ struct olsr_if_weight {
 };
 
 struct if_config_options {
-  union olsr_ip_addr ipv4_broadcast;
+  union olsr_ip_addr ipv4_multicast;
+  union olsr_ip_addr ipv6_multicast;
+
+  union olsr_ip_addr ipv4_src;
+  struct olsr_ip_prefix ipv6_src;
+
   int mode;
-  int ipv6_addrtype;
-  union olsr_ip_addr ipv6_multi_site;
-  union olsr_ip_addr ipv6_multi_glbl;
+
   struct olsr_if_weight weight;
   struct olsr_msg_params hello_params;
   struct olsr_msg_params tc_params;
   struct olsr_msg_params mid_params;
   struct olsr_msg_params hna_params;
   struct olsr_lq_mult *lq_mult;
+  int orig_lq_mult_cnt;
   bool autodetect_chg;
 };
 
 struct olsr_if {
   char *name;
-  char *config;
   bool configured;
   bool host_emul;
   union olsr_ip_addr hemu_ip;
   struct interface *interf;
-  struct if_config_options *cnf;
+  struct if_config_options *cnf, *cnfi;
   struct olsr_if *next;
 };
 
@@ -178,12 +212,6 @@ struct plugin_entry {
   struct plugin_entry *next;
 };
 
-typedef enum {
-  FIBM_FLAT,
-  FIBM_CORRECT,
-  FIBM_APPROX
-} olsr_fib_metric_options;
-
 /*
  * The config struct
  */
@@ -196,9 +224,10 @@ struct olsrd_config {
   int ip_version;
   bool allow_no_interfaces;
   uint16_t tos;
-  uint8_t rtproto;
-  uint8_t rttable;
-  uint8_t rttable_default;
+  uint8_t rt_proto;
+  uint8_t rt_table, rt_table_default, rt_table_tunnel;
+  int32_t rt_table_pri, rt_table_tunnel_pri;
+  int32_t rt_table_defaultolsr_pri, rt_table_default_pri;
   uint8_t willingness;
   bool willingness_auto;
   int ipc_connections;
@@ -208,6 +237,7 @@ struct olsrd_config {
   struct plugin_entry *plugins;
   struct ip_prefix_list *hna_entries;
   struct ip_prefix_list *ipc_nets;
+  struct if_config_options *interface_defaults;
   struct olsr_if *interfaces;
   float pollrate;
   float nic_chgs_pollrate;
@@ -216,28 +246,44 @@ struct olsrd_config {
   uint8_t mpr_coverage;
   uint8_t lq_level;
   uint8_t lq_fish;
-  float lq_dinter;
   float lq_aging;
   char *lq_algorithm;
-  uint8_t lq_dlimit;
+
+  float min_tc_vtime;
+
+  char *lock_file;
+  bool use_niit;
+
+  bool smart_gw_active, smart_gw_allow_nat, smart_gw_uplink_nat;
+  enum smart_gw_uplinktype smart_gw_type;
+  uint32_t smart_gw_uplink, smart_gw_downlink;
+  struct olsr_ip_prefix smart_gw_prefix;
+
+  /* Main address of this node */
+  union olsr_ip_addr main_addr, unicast_src_ip;
+  bool use_src_ip_routes;
 
   /* Stuff set by olsrd */
-  uint16_t system_tick_divider;        /* Tick resolution */
   uint8_t maxplen;                     /* maximum prefix len */
   size_t ipsize;                       /* Size of address */
   bool del_gws;                        /* Delete InternetGWs at startup */
-  union olsr_ip_addr main_addr;        /* Main address of this node */
   float will_int;
   float max_jitter;
   int exit_value;                      /* Global return value for process termination */
   float max_tc_vtime;
 
+  int niit4to6_if_index, niit6to4_if_index;
+
+  /*many potential parameters or helper variables for smartgateway*/
+  bool has_ipv4_gateway, has_ipv6_gateway;
+
   int ioctl_s;                         /* Socket used for ioctl calls */
-#if LINUX_POLICY_ROUTING
+#ifdef LINUX_NETLINK_ROUTING
   int rtnl_s;                          /* Socket used for rtnetlink messages */
+  int rt_monitor_socket;
 #endif
 
-#if defined __FreeBSD__ || defined __MacOSX__ || defined __NetBSD__ || defined __OpenBSD__
+#if defined __FreeBSD__ || defined __FreeBSD_kernel__ || defined __MacOSX__ || defined __NetBSD__ || defined __OpenBSD__
   int rts;                             /* Socket used for route changes on BSDs */
 #endif
   float lq_nat_thresh;
@@ -247,6 +293,10 @@ struct olsrd_config {
 extern "C" {
 #endif
 
+  extern const char *GW_UPLINK_TXT[];
+  extern const char *FIB_METRIC_TXT[];
+  extern const char *OLSR_IF_MODE[];
+
 /*
  * List functions
  */
@@ -261,7 +311,7 @@ extern "C" {
  * Interface to parser
  */
 
-  struct olsrd_config *olsrd_parse_cnf(const char *);
+  int olsrd_parse_cnf(const char *);
 
   int olsrd_sanity_check_cnf(struct olsrd_config *);
 
@@ -269,9 +319,9 @@ extern "C" {
 
   void olsrd_print_cnf(struct olsrd_config *);
 
-  int olsrd_write_cnf(struct olsrd_config *, const char *);
+  void olsrd_write_cnf_autobuf(struct autobuf *out, struct olsrd_config *cnf);
 
-  int olsrd_write_cnf_buf(struct olsrd_config *, char *, uint32_t);
+  int olsrd_write_cnf(struct olsrd_config *, const char *);
 
   struct if_config_options *get_default_if_config(void);
 
