@@ -1,35 +1,35 @@
 
 /*
  * The olsr.org Optimized Link-State Routing daemon(olsrd)
- * Copyright (c) 2004, Andreas Tønnesen(andreto@olsr.org)
+ * Copyright (c) 2004, Andreas Tonnesen(andreto@olsr.org)
  * LSDB rewrite (c) 2007, Hannes Gredler (hannes@gredler.at)
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without 
- * modification, are permitted provided that the following conditions 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
  * are met:
  *
- * * Redistributions of source code must retain the above copyright 
+ * * Redistributions of source code must retain the above copyright
  *   notice, this list of conditions and the following disclaimer.
- * * Redistributions in binary form must reproduce the above copyright 
- *   notice, this list of conditions and the following disclaimer in 
- *   the documentation and/or other materials provided with the 
+ * * Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in
+ *   the documentation and/or other materials provided with the
  *   distribution.
- * * Neither the name of olsr.org, olsrd nor the names of its 
- *   contributors may be used to endorse or promote products derived 
+ * * Neither the name of olsr.org, olsrd nor the names of its
+ *   contributors may be used to endorse or promote products derived
  *   from this software without specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
- * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
- * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
- * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
- * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
- * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+ * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+ * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  *
  * Visit http://www.olsr.org for more information.
@@ -52,12 +52,14 @@
 #include "net_olsr.h"
 #include "lq_plugin.h"
 #include "olsr_cookie.h"
+#include "duplicate_set.h"
+#include "gateway.h"
 
 #include <assert.h>
 
 /* Root of the link state database */
 struct avl_tree tc_tree;
-struct tc_entry *tc_myself;	       /* Shortcut to ourselves */
+struct tc_entry *tc_myself;            /* Shortcut to ourselves */
 
 /* Some cookies for stats keeping */
 struct olsr_cookie_info *tc_edge_gc_timer_cookie = NULL;
@@ -89,38 +91,38 @@ struct olsr_cookie_info *tc_mem_cookie = NULL;
 /* Enlarges the value window for upcoming ansn/seqno to be accepted */
 #define TC_SEQNO_WINDOW_MULT 8
 
-static olsr_bool
-olsr_seq_inrange_low(int beg, int end, olsr_u16_t seq)
+static bool
+olsr_seq_inrange_low(int beg, int end, uint16_t seq)
 {
   if (beg < 0) {
-    if (seq >= (olsr_u16_t) beg || seq < end) {
-      return OLSR_TRUE;
+    if (seq >= (uint16_t) beg || seq < end) {
+      return true;
     }
   } else if (end >= 0x10000) {
-    if (seq >= beg || seq < (olsr_u16_t) end) {
-      return OLSR_TRUE;
+    if (seq >= beg || seq < (uint16_t) end) {
+      return true;
     }
   } else if (seq >= beg && seq < end) {
-    return OLSR_TRUE;
+    return true;
   }
-  return OLSR_FALSE;
+  return false;
 }
 
-static olsr_bool
-olsr_seq_inrange_high(int beg, int end, olsr_u16_t seq)
+static bool
+olsr_seq_inrange_high(int beg, int end, uint16_t seq)
 {
   if (beg < 0) {
-    if (seq > (olsr_u16_t) beg || seq <= end) {
-      return OLSR_TRUE;
+    if (seq > (uint16_t) beg || seq <= end) {
+      return true;
     }
   } else if (end >= 0x10000) {
-    if (seq > beg || seq <= (olsr_u16_t) end) {
-      return OLSR_TRUE;
+    if (seq > beg || seq <= (uint16_t) end) {
+      return true;
     }
   } else if (seq > beg && seq <= end) {
-    return OLSR_TRUE;
+    return true;
   }
-  return OLSR_FALSE;
+  return false;
 }
 
 /**
@@ -143,7 +145,6 @@ olsr_add_tc_entry(union olsr_ip_addr *adr)
   if (ipequal(&olsr_cnf->main_addr, &all_zero)) {
     return NULL;
   }
-
 #ifdef DEBUG
   OLSR_PRINTF(1, "TC: add entry %s\n", olsr_ip_to_string(&buf, adr));
 #endif
@@ -191,16 +192,11 @@ olsr_init_tc(void)
   /*
    * Get some cookies for getting stats to ease troubleshooting.
    */
-  tc_edge_gc_timer_cookie =
-    olsr_alloc_cookie("TC edge GC", OLSR_COOKIE_TYPE_TIMER);
-  tc_validity_timer_cookie =
-    olsr_alloc_cookie("TC validity", OLSR_COOKIE_TYPE_TIMER);
+  tc_edge_gc_timer_cookie = olsr_alloc_cookie("TC edge GC", OLSR_COOKIE_TYPE_TIMER);
+  tc_validity_timer_cookie = olsr_alloc_cookie("TC validity", OLSR_COOKIE_TYPE_TIMER);
 
-  tc_edge_mem_cookie =
-    olsr_alloc_cookie("tc_edge_entry", OLSR_COOKIE_TYPE_MEMORY);
-  olsr_cookie_set_memory_size(tc_edge_mem_cookie,
-			      sizeof(struct tc_edge_entry) +
-			      active_lq_handler->tc_lq_size);
+  tc_edge_mem_cookie = olsr_alloc_cookie("tc_edge_entry", OLSR_COOKIE_TYPE_MEMORY);
+  olsr_cookie_set_memory_size(tc_edge_mem_cookie, sizeof(struct tc_edge_entry) + active_lq_handler->tc_lq_size);
 
   tc_mem_cookie = olsr_alloc_cookie("tc_entry", OLSR_COOKIE_TYPE_MEMORY);
   olsr_cookie_set_memory_size(tc_mem_cookie, sizeof(struct tc_entry));
@@ -211,6 +207,14 @@ olsr_init_tc(void)
   tc_myself = olsr_add_tc_entry(&olsr_cnf->main_addr);
 }
 
+void olsr_delete_all_tc_entries(void) {
+  struct tc_entry *tc;
+
+  OLSR_FOR_ALL_TC_ENTRIES(tc) {
+    olsr_delete_tc_entry(tc);
+  } OLSR_FOR_ALL_TC_ENTRIES_END(tc)
+}
+
 /**
  * The main ip address has changed.
  * Do the needful.
@@ -237,7 +241,7 @@ olsr_change_myself_tc(void)
    * The old entry for ourselves is gone, generate a new one and trigger SPF.
    */
   tc_myself = olsr_add_tc_entry(&olsr_cnf->main_addr);
-  changes_topology = OLSR_TRUE;
+  changes_topology = true;
 }
 
 /*
@@ -281,6 +285,10 @@ olsr_delete_tc_entry(struct tc_entry *tc)
   OLSR_PRINTF(1, "TC: del entry %s\n", olsr_ip_to_string(&buf, &tc->addr));
 #endif
 
+  /* delete gateway if available */
+#ifdef LINUX_NETLINK_ROUTING
+  olsr_delete_gateway_entry(&tc->addr, FORCE_DELETE_GW_ENTRY);
+#endif
   /*
    * Delete the rt_path for ourselves.
    */
@@ -350,12 +358,9 @@ olsr_tc_edge_to_string(struct tc_edge_entry *tc_edge)
   struct tc_entry *tc = tc_edge->tc;
   struct lqtextbuffer lqbuffer1, lqbuffer2;
 
-  snprintf(buf, sizeof(buf),
-	   "%s > %s, cost (%6s) %s",
-	   olsr_ip_to_string(&addrbuf, &tc->addr),
-	   olsr_ip_to_string(&dstbuf, &tc_edge->T_dest_addr),
-	   get_tc_edge_entry_text(tc_edge, '/', &lqbuffer1),
-	   get_linkcost_text(tc_edge->cost, OLSR_FALSE, &lqbuffer2));
+  snprintf(buf, sizeof(buf), "%s > %s, cost (%6s) %s", olsr_ip_to_string(&addrbuf, &tc->addr),
+           olsr_ip_to_string(&dstbuf, &tc_edge->T_dest_addr), get_tc_edge_entry_text(tc_edge, '/', &lqbuffer1),
+           get_linkcost_text(tc_edge->cost, false, &lqbuffer2));
 
   return buf;
 }
@@ -369,12 +374,16 @@ static void
 olsr_expire_tc_entry(void *context)
 {
   struct tc_entry *tc;
+  struct ipaddr_str buf;
 
   tc = (struct tc_entry *)context;
+
+  OLSR_PRINTF(3, "TC: expire node entry %s\n", olsr_ip_to_string(&buf, &tc->addr));
+
   tc->validity_timer = NULL;
 
   olsr_delete_tc_entry(tc);
-  changes_topology = OLSR_TRUE;
+  changes_topology = true;
 }
 
 /**
@@ -386,12 +395,16 @@ static void
 olsr_expire_tc_edge_gc(void *context)
 {
   struct tc_entry *tc;
+  struct ipaddr_str buf;
 
   tc = (struct tc_entry *)context;
+
+  OLSR_PRINTF(3, "TC: expire edge entry %s\n", olsr_ip_to_string(&buf, &tc->addr));
+
   tc->edge_gc_timer = NULL;
 
   if (olsr_delete_outdated_tc_edges(tc)) {
-    changes_topology = OLSR_TRUE;
+    changes_topology = true;
   }
 }
 
@@ -399,10 +412,10 @@ olsr_expire_tc_edge_gc(void *context)
  * If the edge does not have a minimum acceptable link quality
  * set the etx cost to infinity such that it gets ignored during
  * SPF calculation.
- * 
+ *
  * @return 1 if the change of the etx value was relevant
  */
-olsr_bool
+bool
 olsr_calc_tc_edge_entry_etx(struct tc_edge_entry *tc_edge)
 {
   olsr_linkcost old;
@@ -411,13 +424,12 @@ olsr_calc_tc_edge_entry_etx(struct tc_edge_entry *tc_edge)
    * Some sanity check before recalculating the etx.
    */
   if (olsr_cnf->lq_level < 1) {
-    return 0;
+    return false;
   }
 
   old = tc_edge->cost;
   tc_edge->cost = olsr_calc_tc_cost(tc_edge);
-
-  return olsr_is_relevant_costchange(old, tc_edge->cost);
+  return true;
 }
 
 /**
@@ -427,8 +439,7 @@ olsr_calc_tc_edge_entry_etx(struct tc_edge_entry *tc_edge)
  * @return a pointer to the created entry
  */
 struct tc_edge_entry *
-olsr_add_tc_edge_entry(struct tc_entry *tc, union olsr_ip_addr *addr,
-		       olsr_u16_t ansn)
+olsr_add_tc_edge_entry(struct tc_entry *tc, union olsr_ip_addr *addr, uint16_t ansn)
 {
 #ifdef DEBUG
   struct ipaddr_str buf;
@@ -464,15 +475,13 @@ olsr_add_tc_edge_entry(struct tc_entry *tc, union olsr_ip_addr *addr,
   tc_neighbor = olsr_lookup_tc_entry(&tc_edge->T_dest_addr);
   if (tc_neighbor) {
 #ifdef DEBUG
-    OLSR_PRINTF(1, "TC:   found neighbor tc_entry %s\n",
-		olsr_ip_to_string(&buf, &tc_neighbor->addr));
+    OLSR_PRINTF(1, "TC:   found neighbor tc_entry %s\n", olsr_ip_to_string(&buf, &tc_neighbor->addr));
 #endif
 
     tc_edge_inv = olsr_lookup_tc_edge(tc_neighbor, &tc->addr);
     if (tc_edge_inv) {
 #ifdef DEBUG
-      OLSR_PRINTF(1, "TC:   found inverse edge for %s\n",
-		  olsr_ip_to_string(&buf, &tc_edge_inv->T_dest_addr));
+      OLSR_PRINTF(1, "TC:   found inverse edge for %s\n", olsr_ip_to_string(&buf, &tc_edge_inv->T_dest_addr));
 #endif
 
       /*
@@ -533,11 +542,11 @@ olsr_delete_tc_edge_entry(struct tc_edge_entry *tc_edge)
  * @param tc the entry to delete edges from
  * @return TRUE if any destinations were deleted, FALSE if not
  */
-olsr_bool
+bool
 olsr_delete_outdated_tc_edges(struct tc_entry *tc)
 {
   struct tc_edge_entry *tc_edge;
-  olsr_bool retval = OLSR_FALSE;
+  bool retval = false;
 
 #if 0
   OLSR_PRINTF(5, "TC: deleting outdated TC-edge entries\n");
@@ -546,9 +555,10 @@ olsr_delete_outdated_tc_edges(struct tc_entry *tc)
   OLSR_FOR_ALL_TC_EDGE_ENTRIES(tc, tc_edge) {
     if (SEQNO_GREATER_THAN(tc->ansn, tc_edge->ansn)) {
       olsr_delete_tc_edge_entry(tc_edge);
-      retval = OLSR_TRUE;
+      retval = true;
     }
-  } OLSR_FOR_ALL_TC_EDGE_ENTRIES_END(tc, tc_edge);
+  }
+  OLSR_FOR_ALL_TC_EDGE_ENTRIES_END(tc, tc_edge);
 
   return retval;
 }
@@ -562,9 +572,7 @@ olsr_delete_outdated_tc_edges(struct tc_entry *tc)
  * @return 1 if any destinations were deleted 0 if not
  */
 static int
-olsr_delete_revoked_tc_edges(struct tc_entry *tc, olsr_u16_t ansn,
-			     union olsr_ip_addr *lower_border,
-			     union olsr_ip_addr *upper_border)
+olsr_delete_revoked_tc_edges(struct tc_entry *tc, uint16_t ansn, union olsr_ip_addr *lower_border, union olsr_ip_addr *upper_border)
 {
   struct tc_edge_entry *tc_edge;
   int retval = 0;
@@ -573,20 +581,20 @@ olsr_delete_revoked_tc_edges(struct tc_entry *tc, olsr_u16_t ansn,
   OLSR_PRINTF(5, "TC: deleting MPRS\n");
 #endif
 
-  olsr_bool passedLowerBorder = OLSR_FALSE;
+  bool passedLowerBorder = false;
 
   OLSR_FOR_ALL_TC_EDGE_ENTRIES(tc, tc_edge) {
     if (!passedLowerBorder) {
       if (avl_comp_default(lower_border, &tc_edge->T_dest_addr) <= 0) {
-	passedLowerBorder = OLSR_TRUE;
+        passedLowerBorder = true;
       } else {
-	continue;
+        continue;
       }
     }
 
     if (passedLowerBorder) {
       if (avl_comp_default(upper_border, &tc_edge->T_dest_addr) <= 0) {
-	break;
+        break;
       }
     }
 
@@ -594,12 +602,14 @@ olsr_delete_revoked_tc_edges(struct tc_entry *tc, olsr_u16_t ansn,
       olsr_delete_tc_edge_entry(tc_edge);
       retval = 1;
     }
-  } OLSR_FOR_ALL_TC_EDGE_ENTRIES_END(tc, tc_edge);
+  }
+  OLSR_FOR_ALL_TC_EDGE_ENTRIES_END(tc, tc_edge);
 
+  if (retval)
+    changes_topology = true;
   return retval;
 }
 
-
 /**
  * Update an edge registered on an entry.
  * Creates new edge-entries if not registered.
@@ -610,8 +620,7 @@ olsr_delete_revoked_tc_edges(struct tc_entry *tc, olsr_u16_t ansn,
  * @return 1 if entries are added 0 if not
  */
 static int
-olsr_tc_update_edge(struct tc_entry *tc, olsr_u16_t ansn,
-		    const unsigned char **curr, union olsr_ip_addr *neighbor)
+olsr_tc_update_edge(struct tc_entry *tc, uint16_t ansn, const unsigned char **curr, union olsr_ip_addr *neighbor)
 {
   struct tc_edge_entry *tc_edge;
   int edge_change;
@@ -659,14 +668,11 @@ olsr_tc_update_edge(struct tc_entry *tc, olsr_u16_t ansn,
      * Update the etx.
      */
     if (olsr_calc_tc_edge_entry_etx(tc_edge)) {
-      if (tc->msg_hops <= olsr_cnf->lq_dlimit) {
-	edge_change = 1;
-      }
+      edge_change = 1;
     }
 #if DEBUG
     if (edge_change) {
-      OLSR_PRINTF(1, "TC:   chg edge entry %s\n",
-		  olsr_tc_edge_to_string(tc_edge));
+      OLSR_PRINTF(1, "TC:   chg edge entry %s\n", olsr_tc_edge_to_string(tc_edge));
     }
 #endif
 
@@ -707,11 +713,8 @@ olsr_print_tc_table(void)
   struct tc_entry *tc;
   const int ipwidth = olsr_cnf->ip_version == AF_INET ? 15 : 30;
 
-  OLSR_PRINTF(1,
-	      "\n--- %s ------------------------------------------------- TOPOLOGY\n\n"
-	      "%-*s %-*s %-14s  %s\n", olsr_wallclock_string(), ipwidth,
-	      "Source IP addr", ipwidth, "Dest IP addr", "      LQ      ",
-	      "ETX");
+  OLSR_PRINTF(1, "\n--- %s ------------------------------------------------- TOPOLOGY\n\n" "%-*s %-*s %-14s  %s\n",
+              olsr_wallclock_string(), ipwidth, "Source IP addr", ipwidth, "Dest IP addr", "      LQ      ", "ETX");
 
   OLSR_FOR_ALL_TC_ENTRIES(tc) {
     struct tc_edge_entry *tc_edge;
@@ -719,12 +722,9 @@ olsr_print_tc_table(void)
       struct ipaddr_str addrbuf, dstaddrbuf;
       struct lqtextbuffer lqbuffer1, lqbuffer2;
 
-      OLSR_PRINTF(1, "%-*s %-*s %-14s %s\n",
-		  ipwidth, olsr_ip_to_string(&addrbuf, &tc->addr),
-		  ipwidth, olsr_ip_to_string(&dstaddrbuf,
-					     &tc_edge->T_dest_addr),
-		  get_tc_edge_entry_text(tc_edge, '/', &lqbuffer1),
-		  get_linkcost_text(tc_edge->cost, OLSR_FALSE, &lqbuffer2));
+      OLSR_PRINTF(1, "%-*s %-*s %-14s %s\n", ipwidth, olsr_ip_to_string(&addrbuf, &tc->addr), ipwidth,
+                  olsr_ip_to_string(&dstaddrbuf, &tc_edge->T_dest_addr), get_tc_edge_entry_text(tc_edge, '/', &lqbuffer1),
+                  get_linkcost_text(tc_edge->cost, false, &lqbuffer2));
 
     } OLSR_FOR_ALL_TC_EDGE_ENTRIES_END(tc, tc_edge);
   } OLSR_FOR_ALL_TC_ENTRIES_END(tc);
@@ -738,46 +738,40 @@ olsr_print_tc_table(void)
  * @param pointer to lower border ip
  * @param upper border flag
  * @param pointer to upper border ip
- * @result 1 if lower/upper border ip have been set 
+ * @result 1 if lower/upper border ip have been set
  */
 static int
-olsr_calculate_tc_border(olsr_u8_t lower_border,
-			 union olsr_ip_addr *lower_border_ip,
-			 olsr_u8_t upper_border,
-			 union olsr_ip_addr *upper_border_ip)
+olsr_calculate_tc_border(uint8_t lower_border, union olsr_ip_addr *lower_border_ip, uint8_t upper_border,
+                         union olsr_ip_addr *upper_border_ip)
 {
   if (lower_border == 0 && upper_border == 0) {
     return 0;
   }
   if (lower_border == 0xff) {
-    memset(&lower_border_ip, 0, sizeof(lower_border_ip));
+    memset(lower_border_ip, 0, sizeof(lower_border_ip));
   } else {
     int i;
 
     lower_border--;
     for (i = 0; i < lower_border / 8; i++) {
-		lower_border_ip->v6.s6_addr[olsr_cnf->ipsize - i - 1] = 0;
+      lower_border_ip->v6.s6_addr[olsr_cnf->ipsize - i - 1] = 0;
     }
-    lower_border_ip->v6.s6_addr[olsr_cnf->ipsize - lower_border / 8 -
-				       1] &= (0xff << (lower_border & 7));
-    lower_border_ip->v6.s6_addr[olsr_cnf->ipsize - lower_border / 8 -
-				       1] |= (1 << (lower_border & 7));
+    lower_border_ip->v6.s6_addr[olsr_cnf->ipsize - lower_border / 8 - 1] &= (0xff << (lower_border & 7));
+    lower_border_ip->v6.s6_addr[olsr_cnf->ipsize - lower_border / 8 - 1] |= (1 << (lower_border & 7));
   }
 
   if (upper_border == 0xff) {
-    memset(&upper_border_ip, 0xff, sizeof(upper_border_ip));
+    memset(upper_border_ip, 0xff, sizeof(upper_border_ip));
   } else {
     int i;
 
     upper_border--;
 
     for (i = 0; i < upper_border / 8; i++) {
-		upper_border_ip->v6.s6_addr[olsr_cnf->ipsize - i - 1] = 0;
+      upper_border_ip->v6.s6_addr[olsr_cnf->ipsize - i - 1] = 0;
     }
-    upper_border_ip->v6.s6_addr[olsr_cnf->ipsize - upper_border / 8 -
-				       1] &= (0xff << (upper_border & 7));
-    upper_border_ip->v6.s6_addr[olsr_cnf->ipsize - upper_border / 8 -
-				       1] |= (1 << (upper_border & 7));
+    upper_border_ip->v6.s6_addr[olsr_cnf->ipsize - upper_border / 8 - 1] &= (0xff << (upper_border & 7));
+    upper_border_ip->v6.s6_addr[olsr_cnf->ipsize - upper_border / 8 - 1] |= (1 << (upper_border & 7));
   }
   return 1;
 }
@@ -788,35 +782,44 @@ olsr_calculate_tc_border(olsr_u8_t lower_border,
  * If the message is interesting enough, update our edges for it,
  * trigger SPF and finally flood it to all our 2way neighbors.
  *
- * The order for extracting data off the message does matter, 
+ * The order for extracting data off the message does matter,
  * as every call to pkt_get increases the packet offset and
  * hence the spot we are looking at.
  */
-void
-olsr_input_tc(union olsr_message *msg,
-	      struct interface *input_if __attribute__ ((unused)),
-	      union olsr_ip_addr *from_addr)
+bool
+olsr_input_tc(union olsr_message * msg, struct interface * input_if __attribute__ ((unused)), union olsr_ip_addr * from_addr)
 {
   struct ipaddr_str buf;
-  olsr_u16_t size, msg_seq, ansn;
-  olsr_u8_t type, ttl, msg_hops, lower_border, upper_border;
+  uint16_t size, msg_seq, ansn;
+  uint8_t type, ttl, msg_hops, lower_border, upper_border;
   olsr_reltime vtime;
   union olsr_ip_addr originator;
   const unsigned char *limit, *curr;
   struct tc_entry *tc;
+  bool emptyTC;
 
   union olsr_ip_addr lower_border_ip, upper_border_ip;
   int borderSet = 0;
 
   curr = (void *)msg;
   if (!msg) {
-    return;
+    return false;
   }
 
   /* We are only interested in TC message types. */
   pkt_get_u8(&curr, &type);
   if ((type != LQ_TC_MESSAGE) && (type != TC_MESSAGE)) {
-    return;
+    return false;
+  }
+
+  /*
+   * If the sender interface (NB: not originator) of this message
+   * is not in the symmetric 1-hop neighborhood of this node, the
+   * message MUST be discarded.
+   */
+  if (check_neighbor_link(from_addr) != SYM_LINK) {
+    OLSR_PRINTF(2, "Received TC from NON SYM neighbor %s\n", olsr_ip_to_string(&buf, from_addr));
+    return false;
   }
 
   pkt_get_reltime(&curr, &vtime);
@@ -836,45 +839,39 @@ olsr_input_tc(union olsr_message *msg,
 
   tc = olsr_lookup_tc_entry(&originator);
 
+  if (vtime < (olsr_reltime)(olsr_cnf->min_tc_vtime*1000)) {
+	  vtime = (olsr_reltime)(olsr_cnf->min_tc_vtime*1000);
+  }
+
   if (tc && 0 != tc->edge_tree.count) {
-    if (olsr_seq_inrange_high((int)tc->msg_seq - TC_SEQNO_WINDOW,
-			      tc->msg_seq,
-			      msg_seq) &&
-	olsr_seq_inrange_high((int)tc->ansn - TC_ANSN_WINDOW, tc->ansn, ansn)) {
+    if (olsr_seq_inrange_high((int)tc->msg_seq - TC_SEQNO_WINDOW, tc->msg_seq, msg_seq)
+        && olsr_seq_inrange_high((int)tc->ansn - TC_ANSN_WINDOW, tc->ansn, ansn)) {
 
       /*
        * Ignore already seen seq/ansn values (small window for mesh memory)
        */
       if ((tc->msg_seq == msg_seq) || (tc->ignored++ < 32)) {
-	return;
+        return false;
       }
 
-      OLSR_PRINTF(1, "Ignored to much LQTC's for %s, restarting\n",
-		  olsr_ip_to_string(&buf, &originator));
+      OLSR_PRINTF(1, "Ignored to much LQTC's for %s, restarting\n", olsr_ip_to_string(&buf, &originator));
 
-    } else
-      if (!olsr_seq_inrange_high
-	  (tc->msg_seq,
-	   (int)tc->msg_seq + TC_SEQNO_WINDOW * TC_SEQNO_WINDOW_MULT, msg_seq)
-	  || !olsr_seq_inrange_low(tc->ansn,
-				   (int)tc->ansn +
-				   TC_ANSN_WINDOW * TC_ANSN_WINDOW_MULT,
-				   ansn)) {
+    } else if (!olsr_seq_inrange_high(tc->msg_seq, (int)tc->msg_seq + TC_SEQNO_WINDOW * TC_SEQNO_WINDOW_MULT, msg_seq)
+               || !olsr_seq_inrange_low(tc->ansn, (int)tc->ansn + TC_ANSN_WINDOW * TC_ANSN_WINDOW_MULT, ansn)) {
 
       /*
        * Only accept future seq/ansn values (large window for node reconnects).
        * Restart in all other cases. Ignore a single stray message.
        */
       if (!tc->err_seq_valid) {
-	tc->err_seq = msg_seq;
-	tc->err_seq_valid = OLSR_TRUE;
+        tc->err_seq = msg_seq;
+        tc->err_seq_valid = true;
       }
       if (tc->err_seq == msg_seq) {
-	return;
+        return false;
       }
 
-      OLSR_PRINTF(2, "Detected node restart for %s\n",
-		  olsr_ip_to_string(&buf, &originator));
+      OLSR_PRINTF(2, "Detected node restart for %s\n", olsr_ip_to_string(&buf, &originator));
     }
   }
 
@@ -892,21 +889,9 @@ olsr_input_tc(union olsr_message *msg,
   tc->msg_seq = msg_seq;
   tc->ansn = ansn;
   tc->ignored = 0;
-  tc->err_seq_valid = OLSR_FALSE;
+  tc->err_seq_valid = false;
 
-  /*
-   * If the sender interface (NB: not originator) of this message
-   * is not in the symmetric 1-hop neighborhood of this node, the
-   * message MUST be discarded.
-   */
-  if (check_neighbor_link(from_addr) != SYM_LINK) {
-    OLSR_PRINTF(2, "Received TC from NON SYM neighbor %s\n",
-		olsr_ip_to_string(&buf, from_addr));
-    return;
-  }
-
-  OLSR_PRINTF(1, "Processing TC from %s, seq 0x%04x\n",
-	      olsr_ip_to_string(&buf, &originator), tc->msg_seq);
+  OLSR_PRINTF(1, "Processing TC from %s, seq 0x%04x\n", olsr_ip_to_string(&buf, &originator), tc->msg_seq);
 
   /*
    * Now walk the edge advertisements contained in the packet.
@@ -914,9 +899,10 @@ olsr_input_tc(union olsr_message *msg,
 
   limit = (unsigned char *)msg + size;
   borderSet = 0;
+  emptyTC = curr >= limit;
   while (curr < limit) {
     if (olsr_tc_update_edge(tc, ansn, &curr, &upper_border_ip)) {
-      changes_topology = OLSR_TRUE;
+      changes_topology = true;
     }
 
     if (!borderSet) {
@@ -929,16 +915,21 @@ olsr_input_tc(union olsr_message *msg,
    * Calculate real border IPs.
    */
   if (borderSet) {
-    borderSet = olsr_calculate_tc_border(lower_border, &lower_border_ip,
-					 upper_border, &upper_border_ip);
+    borderSet = olsr_calculate_tc_border(lower_border, &lower_border_ip, upper_border, &upper_border_ip);
   }
 
   /*
    * Set or change the expiration timer accordingly.
    */
-  olsr_set_timer(&tc->validity_timer, vtime,
-		 OLSR_TC_VTIME_JITTER, OLSR_TIMER_ONESHOT,
-		 &olsr_expire_tc_entry, tc, tc_validity_timer_cookie->ci_id);
+  olsr_set_timer(&tc->validity_timer, vtime, OLSR_TC_VTIME_JITTER, OLSR_TIMER_ONESHOT, &olsr_expire_tc_entry, tc,
+                 tc_validity_timer_cookie);
+
+  if (emptyTC && lower_border == 0xff && upper_border == 0xff) {
+    /* handle empty TC with border flags 0xff */
+    memset(&lower_border_ip, 0x00, sizeof(lower_border_ip));
+    memset(&upper_border_ip, 0xff, sizeof(upper_border_ip));
+    borderSet = 1;
+  }
 
   if (borderSet) {
 
@@ -952,20 +943,27 @@ olsr_input_tc(union olsr_message *msg,
      * Kick the the edge garbage collection timer. In the meantime hopefully
      * all edges belonging to a multipart neighbor set will arrive.
      */
-    olsr_set_timer(&tc->edge_gc_timer, OLSR_TC_EDGE_GC_TIME,
-		   OLSR_TC_EDGE_GC_JITTER, OLSR_TIMER_ONESHOT,
-		   &olsr_expire_tc_edge_gc, tc, tc_edge_gc_timer_cookie->ci_id);
+    olsr_set_timer(&tc->edge_gc_timer, OLSR_TC_EDGE_GC_TIME, OLSR_TC_EDGE_GC_JITTER, OLSR_TIMER_ONESHOT, &olsr_expire_tc_edge_gc,
+                   tc, tc_edge_gc_timer_cookie);
   }
 
-  /*
-   * Last, flood the message to our other neighbors.
-   */
-  olsr_forward_message(msg, from_addr);
-  return;
+  if (emptyTC && borderSet) {
+    /* cleanup MIDs and HNAs if all edges have been erased by
+     * an empty TC, then alert the duplicate set and kill the
+     * tc entry */
+    olsr_cleanup_mid(&originator);
+    olsr_cleanup_hna(&originator);
+    olsr_cleanup_duplicates(&originator);
+
+    olsr_delete_tc_entry(tc);
+  }
+  /* Forward the message */
+  return true;
 }
 
 /*
  * Local Variables:
  * c-basic-offset: 2
+ * indent-tabs-mode: nil
  * End:
  */
