@@ -39,7 +39,7 @@
  *
  */
 
-#if defined __FreeBSD__ || defined __MacOSX__ || defined __NetBSD__ || defined __OpenBSD__
+#if defined __FreeBSD__ || defined __FreeBSD_kernel__ || defined __MacOSX__ || defined __NetBSD__ || defined __OpenBSD__
 #define ifr_netmask ifr_addr
 #endif
 
@@ -50,7 +50,6 @@
 #include "olsr.h"
 #include "net_os.h"
 #include "net_olsr.h"
-#include "socket_parser.h"
 #include "parser.h"
 #include "scheduler.h"
 #include "generate_msg.h"
@@ -58,6 +57,7 @@
 #include "lq_packet.h"
 #include "log.h"
 #include "link_set.h"
+
 #include <signal.h>
 #include <sys/types.h>
 #include <net/if.h>
@@ -144,7 +144,7 @@ check_interface_updates(void *foo __attribute__ ((unused)))
 int
 chk_if_changed(struct olsr_if *iface)
 {
-  struct interface *ifp, *tmp_ifp;
+  struct interface *ifp;
   struct ifreq ifr;
   struct sockaddr_in6 tmp_saddr6;
   int if_changes;
@@ -180,8 +180,8 @@ chk_if_changed(struct olsr_if *iface)
    * First check if the interface is set DOWN
    */
 
-  if ((ifp->int_flags & IFF_UP) == 0) {
-    OLSR_PRINTF(1, "\tInterface %s not up - removing it...\n", iface->name);
+  if ((ifp->int_flags & IFF_UP) == 0 || (ifp->int_flags & IFF_RUNNING) == 0) {
+    OLSR_PRINTF(1, "\tInterface %s not up and running - removing it...\n", iface->name);
     goto remove_interface;
   }
 
@@ -192,7 +192,7 @@ chk_if_changed(struct olsr_if *iface)
    */
 
   /* Check broadcast */
-  if ((olsr_cnf->ip_version == AF_INET) && !iface->cnf->ipv4_broadcast.v4.s_addr &&     /* Skip if fixed bcast */
+  if ((olsr_cnf->ip_version == AF_INET) && !iface->cnf->ipv4_multicast.v4.s_addr &&     /* Skip if fixed bcast */
       (!(ifp->int_flags & IFF_BROADCAST))) {
     OLSR_PRINTF(3, "\tNo broadcast - removing\n");
     goto remove_interface;
@@ -237,25 +237,19 @@ chk_if_changed(struct olsr_if *iface)
 
   /* IP version 6 */
   if (olsr_cnf->ip_version == AF_INET6) {
-#ifdef DEBUG
     struct ipaddr_str buf;
-#endif
-    /* Get interface address */
-
-    if (get_ipv6_address(ifr.ifr_name, &tmp_saddr6, iface->cnf->ipv6_addrtype) <= 0) {
-      if (iface->cnf->ipv6_addrtype == IPV6_ADDR_SITELOCAL)
-        OLSR_PRINTF(3, "\tCould not find site-local IPv6 address for %s\n", ifr.ifr_name);
-      else
-        OLSR_PRINTF(3, "\tCould not find global IPv6 address for %s\n", ifr.ifr_name);
+    /* Get dst interface address */
 
+    if (0 == get_ipv6_address(iface->name, &tmp_saddr6, (iface->cnf->ipv6_src.prefix_len == 0) ? NULL : (&iface->cnf->ipv6_src))) {
+      OLSR_PRINTF(3, "\tCould not find ip address for %s with prefix %s.\n", ifr.ifr_name, olsr_ip_prefix_to_string(&iface->cnf->ipv6_src));
       goto remove_interface;
     }
+
 #ifdef DEBUG
-    OLSR_PRINTF(3, "\tAddress: %s\n", ip6_to_string(&buf, &tmp_saddr6.sin6_addr));
+    OLSR_PRINTF(3, "\tAddress: %s\n", ip6_to_string(&buf, &iface->cnf->ipv6_multicast.v6));
 #endif
 
     if (memcmp(&tmp_saddr6.sin6_addr, &ifp->int6_addr.sin6_addr, olsr_cnf->ipsize) != 0) {
-      struct ipaddr_str buf;
       OLSR_PRINTF(1, "New IP address for %s:\n", ifr.ifr_name);
       OLSR_PRINTF(1, "\tOld: %s\n", ip6_to_string(&buf, &ifp->int6_addr.sin6_addr));
       OLSR_PRINTF(1, "\tNew: %s\n", ip6_to_string(&buf, &tmp_saddr6.sin6_addr));
@@ -272,9 +266,11 @@ chk_if_changed(struct olsr_if *iface)
       memcpy(&ifp->int6_addr.sin6_addr, &tmp_saddr6.sin6_addr, olsr_cnf->ipsize);
       memcpy(&ifp->ip_addr, &tmp_saddr6.sin6_addr, olsr_cnf->ipsize);
 
-      run_ifchg_cbs(ifp, IFCHG_IF_UPDATE);
+      olsr_trigger_ifchange(ifp->if_index, ifp, IFCHG_IF_UPDATE);
 
-      return 1;
+      /* we have to make sure that olsrd uses the new source address of this interface */
+      olsr_remove_interface(iface); /* so we remove the interface completely */
+      chk_if_up(iface,3); /* and create it again to get new sockets,..*/
     }
     return 0;
 
@@ -292,14 +288,14 @@ chk_if_changed(struct olsr_if *iface)
 #endif
 
     if (memcmp
-        (&((struct sockaddr_in *)&ifp->int_addr)->sin_addr.s_addr, &((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr,
+        (&((struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifp->int_addr))->sin_addr.s_addr, &((struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_addr))->sin_addr.s_addr,
          olsr_cnf->ipsize) != 0) {
       /* New address */
       OLSR_PRINTF(1, "IPv4 address changed for %s\n", ifr.ifr_name);
       OLSR_PRINTF(1, "\tOld:%s\n", ip4_to_string(&buf, ifp->int_addr.sin_addr));
       OLSR_PRINTF(1, "\tNew:%s\n", sockaddr4_to_string(&buf, &ifr.ifr_addr));
 
-      ifp->int_addr = *(struct sockaddr_in *)&ifr.ifr_addr;
+      ifp->int_addr = *(struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_addr);
       /* deactivated to prevent change of originator IP */
 #if 0
       if (memcmp(&olsr_cnf->main_addr, &ifp->ip_addr, olsr_cnf->ipsize) == 0) {
@@ -308,9 +304,12 @@ chk_if_changed(struct olsr_if *iface)
         memcpy(&olsr_cnf->main_addr, &((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr, olsr_cnf->ipsize);
       }
 #endif
-      memcpy(&ifp->ip_addr, &((struct sockaddr_in *)&ifr.ifr_addr)->sin_addr.s_addr, olsr_cnf->ipsize);
+      memcpy(&ifp->ip_addr, &((struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_addr))->sin_addr.s_addr, olsr_cnf->ipsize);
 
-      if_changes = 1;
+      /* we have to make sure that olsrd uses the new source address of this interface */
+      olsr_remove_interface(iface); /* so we remove the interface completely */
+      chk_if_up(iface,3); /* and create it again to get new sockets,..*/
+      return 0;
     }
 
     /* Check netmask */
@@ -323,20 +322,19 @@ chk_if_changed(struct olsr_if *iface)
 #endif
 
     if (memcmp
-        (&((struct sockaddr_in *)&ifp->int_netmask)->sin_addr.s_addr, &((struct sockaddr_in *)&ifr.ifr_netmask)->sin_addr.s_addr,
+        (&((struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifp->int_netmask))->sin_addr.s_addr, &((struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_netmask))->sin_addr.s_addr,
          olsr_cnf->ipsize) != 0) {
-      struct ipaddr_str buf;
       /* New address */
       OLSR_PRINTF(1, "IPv4 netmask changed for %s\n", ifr.ifr_name);
       OLSR_PRINTF(1, "\tOld:%s\n", ip4_to_string(&buf, ifp->int_netmask.sin_addr));
       OLSR_PRINTF(1, "\tNew:%s\n", sockaddr4_to_string(&buf, &ifr.ifr_netmask));
 
-      ifp->int_netmask = *(struct sockaddr_in *)&ifr.ifr_netmask;
+      ifp->int_netmask = *(struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_netmask);
 
       if_changes = 1;
     }
 
-    if (!iface->cnf->ipv4_broadcast.v4.s_addr) {
+    if (!iface->cnf->ipv4_multicast.v4.s_addr) {
       /* Check broadcast address */
       if (ioctl(olsr_cnf->ioctl_s, SIOCGIFBRDADDR, &ifr) < 0) {
         olsr_syslog(OLSR_LOG_ERR, "%s: ioctl (get broadaddr)", ifr.ifr_name);
@@ -346,92 +344,27 @@ chk_if_changed(struct olsr_if *iface)
       OLSR_PRINTF(3, "\tBroadcast address:%s\n", sockaddr4_to_string(&buf, &ifr.ifr_broadaddr));
 #endif
 
-      if (ifp->int_broadaddr.sin_addr.s_addr != ((struct sockaddr_in *)&ifr.ifr_broadaddr)->sin_addr.s_addr) {
-        struct ipaddr_str buf;
+      if (ifp->int_broadaddr.sin_addr.s_addr != ((struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_broadaddr))->sin_addr.s_addr) {
         /* New address */
         OLSR_PRINTF(1, "IPv4 broadcast changed for %s\n", ifr.ifr_name);
         OLSR_PRINTF(1, "\tOld:%s\n", ip4_to_string(&buf, ifp->int_broadaddr.sin_addr));
         OLSR_PRINTF(1, "\tNew:%s\n", sockaddr4_to_string(&buf, &ifr.ifr_broadaddr));
 
-        ifp->int_broadaddr = *(struct sockaddr_in *)&ifr.ifr_broadaddr;
+        ifp->int_broadaddr = *(struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_broadaddr);
         if_changes = 1;
       }
     }
   }
 
   if (if_changes)
-    run_ifchg_cbs(ifp, IFCHG_IF_UPDATE);
+    olsr_trigger_ifchange(ifp->if_index, ifp, IFCHG_IF_UPDATE);
 
   return if_changes;
 
 remove_interface:
-  OLSR_PRINTF(1, "Removing interface %s\n", iface->name);
-  olsr_syslog(OLSR_LOG_INFO, "Removing interface %s\n", iface->name);
-
-  olsr_delete_link_entry_by_ip(&ifp->ip_addr);
-
-  /*
-   *Call possible ifchange functions registered by plugins
-   */
-  run_ifchg_cbs(ifp, IFCHG_IF_REMOVE);
-
-  /* Dequeue */
-  if (ifp == ifnet) {
-    ifnet = ifp->int_next;
-  } else {
-    tmp_ifp = ifnet;
-    while (tmp_ifp->int_next != ifp) {
-      tmp_ifp = tmp_ifp->int_next;
-    }
-    tmp_ifp->int_next = ifp->int_next;
-  }
-
-  /* Remove output buffer */
-  net_remove_buffer(ifp);
-
-  /* Check main addr */
-  /* deactivated to prevent change of originator IP */
-#if 0
-  if (ipequal(&olsr_cnf->main_addr, &ifp->ip_addr)) {
-    if (ifnet == NULL) {
-      /* No more interfaces */
-      memset(&olsr_cnf->main_addr, 0, olsr_cnf->ipsize);
-      OLSR_PRINTF(1, "No more interfaces...\n");
-    } else {
-      struct ipaddr_str buf;
-      olsr_cnf->main_addr = ifnet->ip_addr;
-      OLSR_PRINTF(1, "New main address: %s\n", olsr_ip_to_string(&buf, &olsr_cnf->main_addr));
-      olsr_syslog(OLSR_LOG_INFO, "New main address: %s\n", olsr_ip_to_string(&buf, &olsr_cnf->main_addr));
-    }
-  }
-#endif
-  /*
-   * Deregister functions for periodic message generation
-   */
-  olsr_stop_timer(ifp->hello_gen_timer);
-  olsr_stop_timer(ifp->tc_gen_timer);
-  olsr_stop_timer(ifp->mid_gen_timer);
-  olsr_stop_timer(ifp->hna_gen_timer);
-
-  iface->configured = 0;
-  iface->interf = NULL;
-  /* Close olsr socket */
-  close(ifp->olsr_socket);
-  remove_olsr_socket(ifp->olsr_socket, &olsr_input);
-
-  /* Free memory */
-  free(ifp->int_name);
-  free(ifp);
-
-  if ((ifnet == NULL) && (!olsr_cnf->allow_no_interfaces)) {
-    OLSR_PRINTF(1, "No more active interfaces - exiting.\n");
-    olsr_syslog(OLSR_LOG_INFO, "No more active interfaces - exiting.\n");
-    olsr_cnf->exit_value = EXIT_FAILURE;
-    kill(getpid(), SIGINT);
-  }
 
+  olsr_remove_interface(iface);
   return 0;
-
 }
 
 /**
@@ -454,6 +387,9 @@ add_hemu_if(struct olsr_if *iface)
 
   memset(ifp, 0, sizeof(struct interface));
 
+  /* initialize backpointer */
+  ifp->olsr_if = iface;
+
   iface->configured = true;
   iface->interf = ifp;
 
@@ -476,6 +412,7 @@ add_hemu_if(struct olsr_if *iface)
   memset(&null_addr, 0, olsr_cnf->ipsize);
   if (ipequal(&null_addr, &olsr_cnf->main_addr)) {
     olsr_cnf->main_addr = iface->hemu_ip;
+    olsr_cnf->unicast_src_ip = iface->hemu_ip;
     OLSR_PRINTF(1, "New main address: %s\n", olsr_ip_to_string(&buf, &olsr_cnf->main_addr));
     olsr_syslog(OLSR_LOG_INFO, "New main address: %s\n", olsr_ip_to_string(&buf, &olsr_cnf->main_addr));
   }
@@ -552,7 +489,7 @@ add_hemu_if(struct olsr_if *iface)
   }
 
   /* Register socket */
-  add_olsr_socket(ifp->olsr_socket, &olsr_input_hostemu);
+  add_olsr_socket(ifp->olsr_socket, &olsr_input_hostemu, NULL, NULL, SP_PR_READ);
 
   /*
    * Register functions for periodic message generation
@@ -560,16 +497,16 @@ add_hemu_if(struct olsr_if *iface)
 
   ifp->hello_gen_timer =
     olsr_start_timer(iface->cnf->hello_params.emission_interval * MSEC_PER_SEC, HELLO_JITTER, OLSR_TIMER_PERIODIC,
-                     olsr_cnf->lq_level == 0 ? &generate_hello : &olsr_output_lq_hello, ifp, hello_gen_timer_cookie->ci_id);
+                     olsr_cnf->lq_level == 0 ? &generate_hello : &olsr_output_lq_hello, ifp, hello_gen_timer_cookie);
   ifp->tc_gen_timer =
     olsr_start_timer(iface->cnf->tc_params.emission_interval * MSEC_PER_SEC, TC_JITTER, OLSR_TIMER_PERIODIC,
-                     olsr_cnf->lq_level == 0 ? &generate_tc : &olsr_output_lq_tc, ifp, tc_gen_timer_cookie->ci_id);
+                     olsr_cnf->lq_level == 0 ? &generate_tc : &olsr_output_lq_tc, ifp, tc_gen_timer_cookie);
   ifp->mid_gen_timer =
     olsr_start_timer(iface->cnf->mid_params.emission_interval * MSEC_PER_SEC, MID_JITTER, OLSR_TIMER_PERIODIC, &generate_mid, ifp,
-                     mid_gen_timer_cookie->ci_id);
+                     mid_gen_timer_cookie);
   ifp->hna_gen_timer =
     olsr_start_timer(iface->cnf->hna_params.emission_interval * MSEC_PER_SEC, HNA_JITTER, OLSR_TIMER_PERIODIC, &generate_hna, ifp,
-                     hna_gen_timer_cookie->ci_id);
+                     hna_gen_timer_cookie);
 
   /* Recalculate max topology hold time */
   if (olsr_cnf->max_tc_vtime < iface->cnf->tc_params.emission_interval)
@@ -581,6 +518,8 @@ add_hemu_if(struct olsr_if *iface)
   ifp->valtimes.mid = reltime_to_me(iface->cnf->mid_params.validity_time * MSEC_PER_SEC);
   ifp->valtimes.hna = reltime_to_me(iface->cnf->hna_params.validity_time * MSEC_PER_SEC);
 
+  ifp->mode = iface->cnf->mode;
+
   return 1;
 }
 
@@ -635,13 +574,13 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
 
   ifs.int_flags = ifr.ifr_flags;
 
-  if ((ifs.int_flags & IFF_UP) == 0) {
-    OLSR_PRINTF(debuglvl, "\tInterface not up - skipping it...\n");
+  if ( ( (ifs.int_flags & IFF_UP) == 0) || ( (ifs.int_flags & IFF_RUNNING) == 0) ) {
+    OLSR_PRINTF(debuglvl, "\tInterface not up & running - skipping it...\n");
     return 0;
   }
 
   /* Check broadcast */
-  if ((olsr_cnf->ip_version == AF_INET) && !iface->cnf->ipv4_broadcast.v4.s_addr &&     /* Skip if fixed bcast */
+  if ((olsr_cnf->ip_version == AF_INET) && !iface->cnf->ipv4_multicast.v4.s_addr &&     /* Skip if fixed bcast */
       (!(ifs.int_flags & IFF_BROADCAST))) {
     OLSR_PRINTF(debuglvl, "\tNo broadcast - skipping\n");
     return 0;
@@ -666,16 +605,12 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
   if (olsr_cnf->ip_version == AF_INET6) {
     /* Get interface address */
     struct ipaddr_str buf;
-    if (get_ipv6_address(ifr.ifr_name, &ifs.int6_addr, iface->cnf->ipv6_addrtype) <= 0) {
-      if (iface->cnf->ipv6_addrtype == IPV6_ADDR_SITELOCAL)
-        OLSR_PRINTF(debuglvl, "\tCould not find site-local IPv6 address for %s\n", ifr.ifr_name);
-      else
-        OLSR_PRINTF(debuglvl, "\tCould not find global IPv6 address for %s\n", ifr.ifr_name);
 
+    if (0 == get_ipv6_address(iface->name, &ifs.int6_addr, (iface->cnf->ipv6_src.prefix_len == 0) ? NULL : (&iface->cnf->ipv6_src))) {
+      OLSR_PRINTF(3, "\tCould not find ip address for %s with prefix %s.\n", ifr.ifr_name, olsr_ip_prefix_to_string(&iface->cnf->ipv6_src));
       return 0;
     }
-
-    OLSR_PRINTF(debuglvl, "\tAddress: %s\n", ip6_to_string(&buf, &ifs.int6_addr.sin6_addr));
+    OLSR_PRINTF(debuglvl, "\tAddress: %s\n", ip6_to_string(&buf, &iface->cnf->ipv6_multicast.v6));
 
     /* Multicast */
     memset(&ifs.int6_multaddr, 0, sizeof(ifs.int6_multaddr));
@@ -683,8 +618,7 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
     ifs.int6_multaddr.sin6_flowinfo = htonl(0);
     ifs.int6_multaddr.sin6_scope_id = if_nametoindex(ifr.ifr_name);
     ifs.int6_multaddr.sin6_port = htons(olsr_cnf->olsrport);
-    ifs.int6_multaddr.sin6_addr =
-      (iface->cnf->ipv6_addrtype == IPV6_ADDR_SITELOCAL) ? iface->cnf->ipv6_multi_site.v6 : iface->cnf->ipv6_multi_glbl.v6;
+    ifs.int6_multaddr.sin6_addr =  iface->cnf->ipv6_multicast.v6;
 
 #ifdef __MacOSX__
     ifs.int6_multaddr.sin6_scope_id = 0;
@@ -696,26 +630,29 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
   /* IP version 4 */
   else {
     /* Get interface address (IPv4) */
-    if (ioctl(olsr_cnf->ioctl_s, SIOCGIFADDR, &ifr) < 0) {
-      OLSR_PRINTF(debuglvl, "\tCould not get address of interface - skipping it\n");
-      return 0;
+    if (iface->cnf->ipv4_src.v4.s_addr) {
+      ifs.int_addr.sin_addr = iface->cnf->ipv4_src.v4;
     }
+    else {
+      if (ioctl(olsr_cnf->ioctl_s, SIOCGIFADDR, &ifr) < 0) {
+        OLSR_PRINTF(debuglvl, "\tCould not get address of interface - skipping it\n");
+        return 0;
+      }
 
-    ifs.int_addr = *(struct sockaddr_in *)&ifr.ifr_addr;
-
+      ifs.int_addr = *(struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_addr);
+    }
     /* Find netmask */
-
     if (ioctl(olsr_cnf->ioctl_s, SIOCGIFNETMASK, &ifr) < 0) {
       olsr_syslog(OLSR_LOG_ERR, "%s: ioctl (get netmask)", ifr.ifr_name);
       return 0;
     }
 
-    ifs.int_netmask = *(struct sockaddr_in *)&ifr.ifr_netmask;
+    ifs.int_netmask = *(struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_netmask);
 
     /* Find broadcast address */
-    if (iface->cnf->ipv4_broadcast.v4.s_addr) {
+    if (iface->cnf->ipv4_multicast.v4.s_addr) {
       /* Specified broadcast */
-      memcpy(&((struct sockaddr_in *)&ifs.int_broadaddr)->sin_addr.s_addr, &iface->cnf->ipv4_broadcast.v4, sizeof(uint32_t));
+      memcpy(&((struct sockaddr_in *)&ifs.int_broadaddr)->sin_addr.s_addr, &iface->cnf->ipv4_multicast.v4, sizeof(uint32_t));
     } else {
       /* Autodetect */
       if (ioctl(olsr_cnf->ioctl_s, SIOCGIFBRDADDR, &ifr) < 0) {
@@ -723,15 +660,12 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
         return 0;
       }
 
-      ifs.int_broadaddr = *(struct sockaddr_in *)&ifr.ifr_broadaddr;
+      ifs.int_broadaddr = *(struct sockaddr_in *)ARM_NOWARN_ALIGN(&ifr.ifr_broadaddr);
     }
 
     /* Deactivate IP spoof filter */
-    deactivate_spoof(if_basename(ifr.ifr_name), &ifs, olsr_cnf->ip_version);
-
     /* Disable ICMP redirects */
-    disable_redirects(if_basename(ifr.ifr_name), &ifs, olsr_cnf->ip_version);
-
+    net_os_set_ifoptions(if_basename(ifr.ifr_name), &ifs);
   }
 
   /* Get interface index */
@@ -774,6 +708,68 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
     OLSR_PRINTF(1, "\tMulticast: %s\n", ip6_to_string(&buf, &ifs.int6_multaddr.sin6_addr));
   }
 
+  name_size = strlen(if_basename(ifr.ifr_name)) + 1;
+  ifs.int_name = olsr_malloc(name_size, "Interface update 3");
+  strscpy(ifs.int_name, if_basename(ifr.ifr_name), name_size);
+
+  if (olsr_cnf->ip_version == AF_INET) {
+    /* IP version 4 */
+    ifs.ip_addr.v4 = ifs.int_addr.sin_addr;
+    /*
+     *We create one socket for each interface and bind
+     *the socket to it. This to ensure that we can control
+     *on what interface the message is transmitted
+     */
+
+    ifs.olsr_socket = getsocket(BUFSPACE, &ifs);
+    ifs.send_socket = getsocket(0, &ifs);
+
+    if (ifs.olsr_socket < 0) {
+      fprintf(stderr, "Could not initialize socket... exiting!\n\n");
+      olsr_syslog(OLSR_LOG_ERR, "Could not initialize socket... exiting!\n\n");
+      olsr_cnf->exit_value = EXIT_FAILURE;
+      free(ifs.int_name);
+      kill(getpid(), SIGINT);
+      return 0;
+    }
+    if (ifs.send_socket < 0) {
+      OLSR_PRINTF(1, "Warning, transmission socket could not be initialized. Abort if-up.\n");
+      close (ifs.olsr_socket);
+      free(ifs.int_name);
+      return 0;
+    }
+  } else {
+    /* IP version 6 */
+    ifs.ip_addr.v6 = ifs.int6_addr.sin6_addr;
+
+    /*
+     *We create one socket for each interface and bind
+     *the socket to it. This to ensure that we can control
+     *on what interface the message is transmitted
+     */
+
+    ifs.olsr_socket = getsocket6(BUFSPACE, &ifs);
+    ifs.send_socket = getsocket6(0, &ifs);
+
+    if (ifs.olsr_socket < 0) {
+      fprintf(stderr, "Could not initialize socket... exiting!\n\n");
+      olsr_syslog(OLSR_LOG_ERR, "Could not initialize socket... exiting!\n\n");
+      olsr_cnf->exit_value = EXIT_FAILURE;
+      free(ifs.int_name);
+      kill(getpid(), SIGINT);
+      return 0;
+    }
+    if (ifs.send_socket < 0) {
+      OLSR_PRINTF(1, "Warning, transmission socket could not be initialized. Abort if-up.\n");
+      close (ifs.olsr_socket);
+      free(ifs.int_name);
+      return 0;
+    }
+
+    join_mcast(&ifs, ifs.olsr_socket);
+    join_mcast(&ifs, ifs.send_socket);
+  }
+
   ifp = olsr_malloc(sizeof(struct interface), "Interface update 2");
 
   iface->configured = 1;
@@ -782,6 +778,9 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
   /* XXX bad code */
   memcpy(ifp, &ifs, sizeof(struct interface));
 
+  /* initialize backpointer */
+  ifp->olsr_if = iface;
+
   ifp->immediate_send_tc = (iface->cnf->tc_params.emission_interval < iface->cnf->hello_params.emission_interval);
   if (olsr_cnf->max_jitter == 0) {
     /* max_jitter determines the max time to store to-be-send-messages, correlated with random() */
@@ -789,58 +788,15 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
       ifp->immediate_send_tc ? iface->cnf->tc_params.emission_interval : iface->cnf->hello_params.emission_interval;
   }
 
-  name_size = strlen(if_basename(ifr.ifr_name)) + 1;
   ifp->gen_properties = NULL;
-  ifp->int_name = olsr_malloc(name_size, "Interface update 3");
-  strscpy(ifp->int_name, if_basename(ifr.ifr_name), name_size);
   ifp->int_next = ifnet;
   ifnet = ifp;
 
-  if (olsr_cnf->ip_version == AF_INET) {
-    /* IP version 4 */
-    ifp->ip_addr.v4 = ifp->int_addr.sin_addr;
-    /*
-     *We create one socket for each interface and bind
-     *the socket to it. This to ensure that we can control
-     *on what interface the message is transmitted
-     */
-
-    ifp->olsr_socket = getsocket(BUFSPACE, ifp->int_name);
-
-    if (ifp->olsr_socket < 0) {
-      fprintf(stderr, "Could not initialize socket... exiting!\n\n");
-      olsr_syslog(OLSR_LOG_ERR, "Could not initialize socket... exiting!\n\n");
-      olsr_cnf->exit_value = EXIT_FAILURE;
-      kill(getpid(), SIGINT);
-    }
-
-  } else {
-    /* IP version 6 */
-    ifp->ip_addr.v6 = ifp->int6_addr.sin6_addr;
-
-    /*
-     *We create one socket for each interface and bind
-     *the socket to it. This to ensure that we can control
-     *on what interface the message is transmitted
-     */
-
-    ifp->olsr_socket = getsocket6(BUFSPACE, ifp->int_name);
-
-    join_mcast(ifp, ifp->olsr_socket);
-
-    if (ifp->olsr_socket < 0) {
-      fprintf(stderr, "Could not initialize socket... exiting!\n\n");
-      olsr_syslog(OLSR_LOG_ERR, "Could not initialize socket... exiting!\n\n");
-      olsr_cnf->exit_value = EXIT_FAILURE;
-      kill(getpid(), SIGINT);
-    }
-
-  }
-
   set_buffer_timer(ifp);
 
   /* Register socket */
-  add_olsr_socket(ifp->olsr_socket, &olsr_input);
+  add_olsr_socket(ifp->olsr_socket, &olsr_input, NULL, NULL, SP_PR_READ);
+  add_olsr_socket(ifp->send_socket, &olsr_input, NULL, NULL, SP_PR_READ);
 
 #ifdef linux
   /* Set TOS */
@@ -867,6 +823,7 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
   if (ipequal(&null_addr, &olsr_cnf->main_addr)) {
     struct ipaddr_str buf;
     olsr_cnf->main_addr = ifp->ip_addr;
+    olsr_cnf->unicast_src_ip = ifp->ip_addr;
     OLSR_PRINTF(1, "New main address: %s\n", olsr_ip_to_string(&buf, &olsr_cnf->main_addr));
     olsr_syslog(OLSR_LOG_INFO, "New main address: %s\n", olsr_ip_to_string(&buf, &olsr_cnf->main_addr));
   }
@@ -876,16 +833,16 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
    */
   ifp->hello_gen_timer =
     olsr_start_timer(iface->cnf->hello_params.emission_interval * MSEC_PER_SEC, HELLO_JITTER, OLSR_TIMER_PERIODIC,
-                     olsr_cnf->lq_level == 0 ? &generate_hello : &olsr_output_lq_hello, ifp, hello_gen_timer_cookie->ci_id);
+                     olsr_cnf->lq_level == 0 ? &generate_hello : &olsr_output_lq_hello, ifp, hello_gen_timer_cookie);
   ifp->tc_gen_timer =
     olsr_start_timer(iface->cnf->tc_params.emission_interval * MSEC_PER_SEC, TC_JITTER, OLSR_TIMER_PERIODIC,
-                     olsr_cnf->lq_level == 0 ? &generate_tc : &olsr_output_lq_tc, ifp, tc_gen_timer_cookie->ci_id);
+                     olsr_cnf->lq_level == 0 ? &generate_tc : &olsr_output_lq_tc, ifp, tc_gen_timer_cookie);
   ifp->mid_gen_timer =
     olsr_start_timer(iface->cnf->mid_params.emission_interval * MSEC_PER_SEC, MID_JITTER, OLSR_TIMER_PERIODIC, &generate_mid, ifp,
-                     mid_gen_timer_cookie->ci_id);
+                     mid_gen_timer_cookie);
   ifp->hna_gen_timer =
     olsr_start_timer(iface->cnf->hna_params.emission_interval * MSEC_PER_SEC, HNA_JITTER, OLSR_TIMER_PERIODIC, &generate_hna, ifp,
-                     hna_gen_timer_cookie->ci_id);
+                     hna_gen_timer_cookie);
 
   /* Recalculate max topology hold time */
   if (olsr_cnf->max_tc_vtime < iface->cnf->tc_params.emission_interval) {
@@ -897,10 +854,12 @@ chk_if_up(struct olsr_if *iface, int debuglvl __attribute__ ((unused)))
   ifp->valtimes.mid = reltime_to_me(iface->cnf->mid_params.validity_time * MSEC_PER_SEC);
   ifp->valtimes.hna = reltime_to_me(iface->cnf->hna_params.validity_time * MSEC_PER_SEC);
 
+  ifp->mode = iface->cnf->mode;
+
   /*
    *Call possible ifchange functions registered by plugins
    */
-  run_ifchg_cbs(ifp, IFCHG_IF_ADD);
+  olsr_trigger_ifchange(ifp->if_index, ifp, IFCHG_IF_ADD);
 
   return 1;
 }
